# --COPYRIGHT--,BSD
#  Copyright (c) $(CPYYEAR), Texas Instruments Incorporated
#  All rights reserved.
# 
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
#  are met:
# 
#  *  Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
# 
#  *  Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
# 
#  *  Neither the name of Texas Instruments Incorporated nor the names of
#     its contributors may be used to endorse or promote products derived
#     from this software without specific prior written permission.
# 
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
#  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
#  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
#  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
#  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
#  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
#  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
#  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
#  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# --/COPYRIGHT--

#------------------------------------------------------------------------------
# DVSDK SOFTWARE COMPONENTS:
#
#    Set the locations of the DVSDK software components. Software component
#    locations are specified with an environment variable (e.g. CE_INSTALL_DIR).
#    This variable is typically set to be the most recent version of the
#    software supported by DMAI.
#
#    Each platform has it's own set of DVSDK software components, and the
#    current platform is set using the PLATFORM variable. See the release
#    notes for currently supported platforms, and the user's guide for more
#    information on how to build DMAI.
#
#    Available variables (append _<PLATFORM> at the end of each name):
#
#    CE_INSTALL_DIR          - Points to the installation directory of Codec
#                              Engine. This directory should have a packages/
#                              directory inside.
#    CODEC_INSTALL_DIR       - Points to the installation directory of
#                              multimedia codecs. This directory should have a
#                              packages/ directory inside.
#    LINK_INSTALL_DIR        - Points to the installation directory of DSP Link.
#                              This directory should have a packages/ directory
#                              inside.
#    CMEM_INSTALL_DIR        - Points to the installation directory of CMEM.
#                              This directory should have a packages/ directory
#                              inside.
#    FC_INSTALL_DIR          - Points to the installation directory of Framework
#                              Components. This directory should have a
#                              packages/ directory inside.
#    LPM_INSTALL_DIR         - Points to the installation directory of the Local
#                              Power Manager. This directory should have a
#                              packages/ directory inside.
#    XDAIS_INSTALL_DIR       - Points to the installation directory of the XDAIS
#                              Algorithm Standard. This directory should have a
#                              packages/ directory inside.
#    BIOS_INSTALL_DIR        - Points to the installation directory of the
#                              DSP/BIOS Real Time Operating System. This
#                              directory should have a packages/ directory
#                              inside.
#    LINUXLIBS_INSTALL_DIR   - Points to the installation directory of 
#                              additional linux libraries needed. This
#                              directory should have usr/, lib/ and include/
#                              directories inside.
#    LINUXKERNEL_INSTALL_DIR - Points to the installation directory of the 
#                              Linux kernel. This directory should have
#                              a Documentation/ directory inside.
#    CROSS_COMPILE           - Points to the GCC cross compilation tool chain.
#                              This variable contains the full path including
#                              the prefix up to the actual "gcc" command.
#    XDC_INSTALL_DIR         - Points to the installation directory of the RTSC
#                              tools. This directory should have a packages/
#                              directory inside.
#    EDMA3_LLD_INSTALL_DIR   - Points to where the EDMA3 Low Level Driver
#                              is installed. This directory should have a
#                              packages/ directory inside.
#    CODEGEN_INSTALL_DIR     - Points to where the TI c6x codegen tools are
#                              installed. This directory should have bin/,
#                              lib/ and include/ directories inside.
#    BIOSUTILS_INSTALL_DIR   - Points to where the DSP/BIOS utilities are
#                              installed. This directory should have a
#                              packages/ directory inside.
#    EXEC_DIR                - Points to where the DMAI applications should
#                              be installed when 'make install' is executed.
#                              Typically this should point to your NFS mounted
#                              target file system for Linux targets.
#
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
# The platform to build DMAI for, see release notes for supported platforms.
# Note that each platform override designation contains a suffix such as
# "_al" or "_db".  This is to designate the operating system environment
# for that platform.  The following operating system environments are
# currently supported:
#
#   al:  ARM/Linux
#   db:  DSP/BIOS
#
# This designation is required as some ARM+DSP platforms may support running
# DMAI under multiple environments.
#------------------------------------------------------------------------------
PLATFORM=o3530_al

#------------------------------------------------------------------------------
# DVSDK installation directories. These are for convenience only and are
# not used outside of this file.
#------------------------------------------------------------------------------
DVSDK_2_00_INSTALL_DIR = /library/dvsdk/dvsdk_2_00_00_16
DVSDK_2_05_INSTALL_DIR = /library/dvsdk/dvsdk_2_05_00_14
DVSDK_3_00_INSTALL_DIR = /library/dvsdk/dvsdk_3_00_00_29

#------------------------------------------------------------------------------
# OMAP 3530 ARM Linux components.
#------------------------------------------------------------------------------
CE_INSTALL_DIR_o3530_al = \
    $(DVSDK_3_00_INSTALL_DIR)/codec_engine_2_20_01
CODEC_INSTALL_DIR_o3530_al = \
    $(DVSDK_3_00_INSTALL_DIR)/omap3530_dvsdk_combos_3_16
LINK_INSTALL_DIR_o3530_al = \
    $(DVSDK_3_00_INSTALL_DIR)/dsplink_1.51
CMEM_INSTALL_DIR_o3530_al = \
    $(DVSDK_3_00_INSTALL_DIR)/linuxutils_2_20
FC_INSTALL_DIR_o3530_al = \
    $(DVSDK_3_00_INSTALL_DIR)/framework_components_2_20_01
LPM_INSTALL_DIR_o3530_al = \
    $(DVSDK_3_00_INSTALL_DIR)/local_power_manager_1_20_01
XDAIS_INSTALL_DIR_o3530_al = \
    $(DVSDK_3_00_INSTALL_DIR)/xdais_6_20
LINUXLIBS_INSTALL_DIR_o3530_al = \
    $(DVSDK_3_00_INSTALL_DIR)/linuxlibs
LINUXKERNEL_INSTALL_DIR_o3530_al = \
    /library/psp/REL_OMAP35x_02.01.00.02/PSP_02.01.00.02/src/linux-02.01.00.02
CROSS_COMPILE_o3530_al = \
    /library/cs/arm-2007q3/bin/arm-none-linux-gnueabi-
XDC_INSTALL_DIR_o3530_al = \
    /library/xdc/xdctools_3_10_03
EXEC_DIR_o3530_al = \
    $(HOME)/workdir/filesys/opt/dvsdk

#------------------------------------------------------------------------------
# DM357 ARM Linux components.
#------------------------------------------------------------------------------
CE_INSTALL_DIR_dm357_al = \
    $(DVSDK_2_05_INSTALL_DIR)/codec_engine_2_21
LINK_INSTALL_DIR_dm357_al = \
    $(DVSDK_2_05_INSTALL_DIR)/dsplink-1_60-prebuilt
CMEM_INSTALL_DIR_dm357_al = \
    $(DVSDK_2_05_INSTALL_DIR)/linuxutils_2_21
FC_INSTALL_DIR_dm357_al = \
    $(DVSDK_2_05_INSTALL_DIR)/framework_components_2_21
XDAIS_INSTALL_DIR_dm357_al = \
    $(DVSDK_2_05_INSTALL_DIR)/xdais_6_21
LINUXKERNEL_INSTALL_DIR_dm357_al = \
    $(HOME)/davincigit/linux-davinci/
CODEC_INSTALL_DIR_dm357_al = \
    $(DVSDK_2_05_INSTALL_DIR)/dm357_codecs_1_00_004
CROSS_COMPILE_dm357_al = \
    /library/mvl/mv_pro_5.0/montavista/pro/devkit/arm/v5t_le/bin/arm_v5t_le-
XDC_INSTALL_DIR_dm357_al = \
    /library/xdc/xdctools_3_10_03
EXEC_DIR_dm357_al = \
    $(HOME)/workdir/filesys/opt/dvsdk

#------------------------------------------------------------------------------
# DM355 ARM Linux components.
#------------------------------------------------------------------------------
CE_INSTALL_DIR_dm355_al = \
    $(DVSDK_2_00_INSTALL_DIR)/codec_engine_2_21
LINK_INSTALL_DIR_dm355_al = \
    $(DVSDK_2_00_INSTALL_DIR)/dsplink-1_60-prebuilt
CMEM_INSTALL_DIR_dm355_al = \
    $(DVSDK_2_00_INSTALL_DIR)/linuxutils_2_21
FC_INSTALL_DIR_dm355_al = \
    $(DVSDK_2_00_INSTALL_DIR)/framework_components_2_21
XDAIS_INSTALL_DIR_dm355_al = \
    $(DVSDK_2_00_INSTALL_DIR)/xdais_6_21
LINUXKERNEL_INSTALL_DIR_dm355_al = \
    $(HOME)/davincigit/linux-davinci/
CROSS_COMPILE_dm355_al = \
    /library/mvl/mv_pro_5.0/montavista/pro/devkit/arm/v5t_le/bin/arm_v5t_le-
CODEC_INSTALL_DIR_dm355_al = \
    $(DVSDK_2_00_INSTALL_DIR)/dm355_codecs_1_13_000
USER_XDC_PATH_dm355_al = \
    $(CE_INSTALL_DIR)/examples
XDC_INSTALL_DIR_dm355_al = \
    /library/xdc/xdctools_3_10_03
EXEC_DIR_dm355_al = \
    $(HOME)/workdir/filesys/opt/dvsdk

#------------------------------------------------------------------------------
# DM6446 ARM Linux components.
#------------------------------------------------------------------------------
CE_INSTALL_DIR_dm6446_al = \
    $(DVSDK_2_00_INSTALL_DIR)/codec_engine_2_21
LINK_INSTALL_DIR_dm6446_al = \
    $(DVSDK_2_00_INSTALL_DIR)/dsplink-1_60-prebuilt
CMEM_INSTALL_DIR_dm6446_al = \
    $(DVSDK_2_00_INSTALL_DIR)/linuxutils_2_21
FC_INSTALL_DIR_dm6446_al = \
    $(DVSDK_2_00_INSTALL_DIR)/framework_components_2_21
XDAIS_INSTALL_DIR_dm6446_al = \
    $(DVSDK_2_00_INSTALL_DIR)/xdais_6_21
LINUXKERNEL_INSTALL_DIR_dm6446_al = \
    $(HOME)/davincigit/linux-davinci/
CROSS_COMPILE_dm6446_al = \
    /library/mvl/mv_pro_5.0/montavista/pro/devkit/arm/v5t_le/bin/arm_v5t_le-
CODEC_INSTALL_DIR_dm6446_al = \
    $(DVSDK_2_00_INSTALL_DIR)/dm6446_dvsdk_combos_2_02
XDC_INSTALL_DIR_dm6446_al = \
    /library/xdc/xdctools_3_10_03
EXEC_DIR_dm6446_al = \
    $(HOME)/workdir/filesys/opt/dvsdk

#------------------------------------------------------------------------------
# DM6467 ARM Linux components.
#------------------------------------------------------------------------------
CE_INSTALL_DIR_dm6467_al = \
    $(DVSDK_2_00_INSTALL_DIR)/codec_engine_2_21
LINK_INSTALL_DIR_dm6467_al = \
    $(DVSDK_2_00_INSTALL_DIR)/dsplink-1_60-prebuilt
CMEM_INSTALL_DIR_dm6467_al = \
    $(DVSDK_2_00_INSTALL_DIR)/linuxutils_2_21
FC_INSTALL_DIR_dm6467_al = \
    $(DVSDK_2_00_INSTALL_DIR)/framework_components_2_21
XDAIS_INSTALL_DIR_dm6467_al = \
    $(DVSDK_2_00_INSTALL_DIR)/xdais_6_21
LINUXKERNEL_INSTALL_DIR_dm6467_al = \
    $(HOME)/davincigit/linux-davinci/
CROSS_COMPILE_dm6467_al = \
    /library/mvl/mv_pro_5.0/montavista/pro/devkit/arm/v5t_le/bin/arm_v5t_le-
CODEC_INSTALL_DIR_dm6467_al = \
    $(DVSDK_2_00_INSTALL_DIR)/dm6467_dvsdk_combos_2_02
XDC_INSTALL_DIR_dm6467_al = \
    /library/xdc/xdctools_3_10_03
EXEC_DIR_dm6467_al = \
    $(HOME)/workdir/filesys/opt/dvsdk

#------------------------------------------------------------------------------
# DM6467 DSP/BIOS components.
#------------------------------------------------------------------------------
CE_INSTALL_DIR_dm6467_db = \
    $(DVSDK_2_00_INSTALL_DIR)/codec_engine_2_21
LINK_INSTALL_DIR_dm6467_db = \
    $(DVSDK_2_00_INSTALL_DIR)/dsplink-1_60-prebuilt
CMEM_INSTALL_DIR_dm6467_db = \
    $(DVSDK_2_00_INSTALL_DIR)/linuxutils_2_21
FC_INSTALL_DIR_dm6467_db = \
    $(DVSDK_2_00_INSTALL_DIR)/framework_components_2_21
BIOSUTILS_INSTALL_DIR_dm6467_db = \
    $(DVSDK_2_00_INSTALL_DIR)/biosutils_1_01_00
XDAIS_INSTALL_DIR_dm6467_db = \
    $(DVSDK_2_00_INSTALL_DIR)/xdais_6_21
BIOS_INSTALL_DIR_dm6467_db = \
    /library/dspbios/bios_5_33_02
EDMA3_LLD_INSTALL_DIR_dm6467_db = \
    $(DVSDK_2_00_INSTALL_DIR)/edma3_lld_1_05_00
CODEC_INSTALL_DIR_dm6467_db = \
    $(DVSDK_2_00_INSTALL_DIR)/dm6467_dvsdk_combos_2_02
CODEGEN_INSTALL_DIR_dm6467_db = \
    /library/ticgt/cg6x_6_0_16
XDC_INSTALL_DIR_dm6467_db = \
    /library/xdc/xdctools_3_10_03
EXEC_DIR_dm6467_db = \
    $(HOME)/workdir/filesys/opt/dvsdk

#------------------------------------------------------------------------------
# DM6446 DSP/BIOS components.
#------------------------------------------------------------------------------
CE_INSTALL_DIR_dm6446_db = \
    $(DVSDK_2_00_INSTALL_DIR)/codec_engine_2_21
LINK_INSTALL_DIR_dm6446_db = \
    $(DVSDK_2_00_INSTALL_DIR)/dsplink-1_60-prebuilt
CMEM_INSTALL_DIR_dm6446_db = \
    $(DVSDK_2_00_INSTALL_DIR)/linuxutils_2_21
FC_INSTALL_DIR_dm6446_db = \
    $(DVSDK_2_00_INSTALL_DIR)/framework_components_2_21
BIOSUTILS_INSTALL_DIR_dm6446_db = \
    $(DVSDK_2_00_INSTALL_DIR)/biosutils_1_01_00
XDAIS_INSTALL_DIR_dm6446_db = \
    $(DVSDK_2_00_INSTALL_DIR)/xdais_6_21
BIOS_INSTALL_DIR_dm6446_db = \
    /library/dspbios/bios_5_33_02
EDMA3_LLD_INSTALL_DIR_dm6446_db = \
    $(DVSDK_2_00_INSTALL_DIR)/edma3_lld_1_05_00
CODEC_INSTALL_DIR_dm6446_db = \
    $(DVSDK_2_00_INSTALL_DIR)/dm6467_dvsdk_combos_2_02
CODEGEN_INSTALL_DIR_dm6446_db = \
    /library/ticgt/cg6x_6_0_16
CODEC_INSTALL_DIR_dm6446_db = \
    $(DVSDK_2_00_INSTALL_DIR)/dm6446_dvsdk_combos_2_02
XDC_INSTALL_DIR_dm6446_db = \
    /library/xdc/xdctools_3_10_03
EXEC_DIR_dm6446_db = \
    $(HOME)/workdir/filesys/opt/dvsdk

#------------------------------------------------------------------------------
# DM6437 DSP/BIOS components.
#------------------------------------------------------------------------------
CE_INSTALL_DIR_dm6437_db = \
    $(DVSDK_2_00_INSTALL_DIR)/codec_engine_2_21
LINK_INSTALL_DIR_dm6437_db = \
    $(DVSDK_2_00_INSTALL_DIR)/dsplink-1_60-prebuilt
CMEM_INSTALL_DIR_dm6437_db = \
    $(DVSDK_2_00_INSTALL_DIR)/linuxutils_2_21
FC_INSTALL_DIR_dm6437_db = \
    $(DVSDK_2_00_INSTALL_DIR)/framework_components_2_21
BIOSUTILS_INSTALL_DIR_dm6437_db = \
    $(DVSDK_2_00_INSTALL_DIR)/biosutils_1_01_00
XDAIS_INSTALL_DIR_dm6437_db = \
    $(DVSDK_2_00_INSTALL_DIR)/xdais_6_21
BIOS_INSTALL_DIR_dm6437_db = \
    /library/dspbios/bios_5_33_02
EDMA3_LLD_INSTALL_DIR_dm6437_db = \
    $(DVSDK_2_00_INSTALL_DIR)/edma3_lld_1_05_00
CODEC_INSTALL_DIR_dm6437_db = \
    $(DVSDK_2_00_INSTALL_DIR)/dm6467_dvsdk_combos_2_02
CODEGEN_INSTALL_DIR_dm6437_db = \
    /library/ticgt/cg6x_6_0_16
CODEC_INSTALL_DIR_dm6437_db = \
    $(DVSDK_2_00_INSTALL_DIR)/dm6446_dvsdk_combos_2_02
XDC_INSTALL_DIR_dm6437_db = \
    /library/xdc/xdctools_3_10_03
EXEC_DIR_dm6437_db = \
    $(HOME)/workdir/filesys/opt/dvsdk
