/*! @page  release_notes Davinci Multimedia Application Interface (DMAI) $version Release Notes

<h2>General</h2>

This DMAI release is targetted to the DaVinci and OMAP platforms, implementing a thin utility layer on top of the operating system (Linux or DSP/BIOS) and the Codec Engine (CE) to assist in quickly writing portable DaVinci and OMAP applications. This document describes the current feature set and its dependencies. A history of changes added for each incremental release is included at the end of the document. 

See the <a href="http://wiki.davincidsp.com/index.php?title=DMAI">DaVinci Wiki</a> for more information on DMAI, including known issues with released versions.

The compatibility information provided in this document assumes you are following the <a href="user_guide.html#mozTocId467933">DMAI Conventions</a> when implementing your application.

See the <a href="user_guide.html#mozTocId467904">Rebuilding DMAI</a> section of the @ref user_guide for instructions on how to build DMAI.

See the <a href="user_guide.html#mozTocId667910">DMAI Reference Applications</a> section of the user guide for information about running the sample applications and writing your own.

<h2>Features</h2>
<ul>
  <li>Currently supported platforms:</li>
  <ul>
     <li>DM6467 ARM Linux (@e dm6467_al build target)</li>
     <li>DM355 ARM Linux (@e dm355_al build target)</li>
     <li>DM357 ARM Linux (@e dm357_al build target)</li>
     <li>DM6446 ARM Linux (@e dm6446_al build target)</li>
     <li>OMAP3530 ARM Linux (@e o3530_al build target)</li>
     <li>DM6446 DSP DSP/BIOS (@e dm6446_db build target)</li>
     <li>DM6467 DSP DSP/BIOS (@e dm6467_db build target)</li>
     <li>DM6437 DSP DSP/BIOS (@e dm6437_db build target)</li>
  </ul>
</ul>

<h2>Dependencies</h2>

All platforms depend on version <a href="https://www-a.ti.com/downloads/sds_support/targetcontent/rtsc/xdctools_3_10_03/index.html">3.10.03</a> of the XDC tools.

<h3>OMAP3530 ARM Linux Dependencies</h3>
DMAI support for OMAP3530 ARM Linux has been developed and tested with components from the DVSDK 3.00.00.29 (build 29) a.k.a. beta2, including Linux Support Package 1.0.2. The code sourcery tool chain is required and downloadable from <a href="http://www.codesourcery.com/gnu_toolchains/arm/portal/package1786?@template=release">here</a>.

<h3>DM357 ARM Linux Dependencies</h3>
DMAI support for DM357 ARM Linux has been developed and tested with components
from the DVSDK 2.05.00.14 (build 14) including Linux Support Package 2.00.00.100, but should work with any later build.

<h3>DM355, DM6446 and DM6467 ARM Linux Dependencies</h3>
DMAI support for DM355, DM6446 and DM6467 ARM Linux has been developed and tested with components from the DVSDK 2.00.00.16 (build 16) including Linux Support Package 2.00.00.090, but should work with any later build.

@b Note! The DM355 MPEG4 Decoder does not support the XDM_FLUSH command, therefore, when used with the video_decode_io2 application, a segmentation fault is generated for the last frame.

<h3>DM6446, DM6467 and DM6437 DSP DSP/BIOS Dependencies</h3>
DMAI support has been developed and tested against the components from the DVSDK 2.00.00.16.

If DMAI is compiled inside the DVSDK (see <a href="user_guide.html#mozTocId467904">Rebuilding DMAI</a>), make sure the @c PLATFORM variable in the top level DVSDK @e Rules.make is set before building any DSP targets.

Make sure the @c BIOS_INSTALL_DIR variable is set correctly in the DVSDK @e Rules.make file. It's easily overlooked as it's not used when building ARM applications.

For DM6437 target, the CODEC_INSTALL_DIR is set to point to the DM6446 codec combos. All DM6446 codec packages,  EXCEPT the video encoders which use the DM6446 hardware accelerator (h264 and mpeg4), will work on DM6437. The DM6437 video encoders can be obtained for free from the:<a href="http://www.ti-estore.com/merchant2/merchant.mvc?Screen=CTGY&Category_Code=dEmbSoft">TI eStore</a>.

<h2>Changelog</h2>

<h3>Since 1.16.00</h3>
<h4>New features</h4>
<ul>
    <li>Support for Montavista Linux 5.0 (and no longer tested against MVL 4.0 and associated LSP releases)</li>
    <li>Support for OMAP3530 against DVSDK 3.00 beta2.</li>
</ul>
<h4>Defects addressed</h4>
<ul>
  <li>@b SDOCM00046005 DMAI sample apps to have command line parameter for selecting video output. <i>Added the -O flag to apps with display output</i>.</li>
  <li>@b SDOCM00051436 DMAI doesn't support all the color spaces that DM355 imaging codecs do. <i>#ColorSpace_YUV444P and ColorSpace_GRAY colorspaces added and support for these added to imaging apps.</i>.</li>
  <li>@b SDOCM00051634 Resizer module should configure the driver as per the Input buffer colorspace for dm6467. <i>Now autodetects the color space and sets 420P or 422P accordingly.</i></li>
  <li>@b SDOCM00051789 DMAI Image decode: The color format of a decoded image should be preserved for the user in imaging apps. <i>Done</i>.</li>
  <li>@b SDOCM00051936 Loader module should expose Memory_Params for ring buffer to allow for customizations such as caching. <i>See #Loader_Attrs.mParams. Note that any alignment will now have to be set up by the user (see sample apps for examples).</i></li>
  <li>@b SDOCM00051937 DMAI sample codec applications should allow for buffers to be allocated cached. <i>When --cache is used in apps, not only will cache maintenance be made for the buffers, but they will also be allocated cached.</i></li>
  <li>@b SDOCM00051938 DMAI apps should allocate buffer sizes as a multiple of the cache line size. <i>Done, see #Dmai_roundUp.</i></li>
  <li>@b SDOCM00053444 DM6467 needs accelerated framecopy. <i>Implemented using the VDCE.</i></li>
  <li>@b SDOCM00053449 video_decode_io2 - the output does not display correct frame number in the output message. <i>Fixed.</i></li>
</ul>

<h4>Compatibility breaks</h4>
N/A

<h3>Since 1.13.00</h3>
<h4>New features</h4>
<ul>
    <li>Added support for <a href="user_guide.html#mozTocId46951">Buffer References</a>, which allows buffer descriptors to manage buffers not allocated by DMAI.</li>
    <li>Most VISA classes, XDM 0.9 and XDM 1.x, are now supported with sample applications.</li>
    <li>DMAI is now released under the BSD license, see @ref license.
</ul>
<h4>Defects addressed</h4>
<ul>
  <li>@b SDOCM00046016 DMAI display module doesn't support using the currently selected display output. <i>Added #Display_Output_SYSTEM for #Display_Attrs.videoOutput to read current setting from SysFS.</i></li>
  <li>@b SDOCM00046067 Request for Fifo to support thresholds on the number of items it contains. <i>Added support for count = #Rendezvous_INFINITE, in which case a force/forceAndReset is necessary to clear the Rendezvous.</i></li>
  <li>@b SDOCM00046113 DMAI apps for DM355 should configure DMAN3. <i>Done</i></li>
  <li>@b SDOCM00046141 Printing of extended errors returned by codecs. <i>Using DMAI_DEBUG=2 (see <a href="user_guide.html#mozTocId467911">Debugging a DMAI application</a>) the user consistently gets extended errors printed out when a failure occurs</i>.</li>
  <li>@b SDOCM00046209 Application examples should set alignment of buffers to 128 when cache is turned on. <i>Added to all *_io applications</i></li> 
  <li>@b SDOCM00048587 BufTab_expand() and BufTab_chunk() cause subsequent calls to Buffer_getBufTab() to return NULL for buffer members. <i>hBufTab is now set correctly when functions create new buffer objects.</i></li>
  <li>@b SDOCM00050087 Need Rendezvous_force() and Rendezvous_reset() in a single operation. <i>Implemented #Rendezvous_forceAndReset method to implement both functions in the same piece of critical code.</i></li>
  <li>@b SDOCM00050657 Capture_create() should allow for skipping of video standard auto detection. <i>Added #Capture_Attrs.videoStd and #Capture_Attrs.decoderIdx to be able to force a video standard and video decoder (dm6467) and skip auto detection.</i></li> 
  <li>@b SDOCM00050963 Must set VIDENC1_DynamicParams.interFrameInterval=1as default. <i>Done</i></li>
  <li>@b SDOCM00051123 Building DMAI inside DVSDK 1.40 for DSP/BIOS targets fail. <i>Added workaround to release notes</i></li>
  <li>@b SDOCM00051141 printf's need to avoid using %lu since it doesn't show up right on DSP targets. <i>Done</i></li>
  <li>@b SDOCM00051273 Loader module should align buffer and make buffer size a multiple of cache line size. <i>Done</i></li>
</ul>
<h4>Compatibility breaks</h4>
N/A

<h3>Since 1.10.00</h3>
<h4>New features</h4>
<ul>
    <li>Introduced support for the DM357 ARM Linux platform. This platforms uses the same peripherals as dm6446, hence this code is shared.</li>
    <li>Added imaging support for xDM 0.9 (@ref ti_sdo_dmai_Ienc and @ref ti_sdo_dmai_Idec) and xDM 1.x (@ref ti_sdo_dmai_Ienc1 and @ref ti_sdo_dmai_Idec1).</li>
    <li>Added #Capture_Attrs.smoothPad to allow for extra lines to be captured while cropping on dm6446 and dm357. This is necessary when using the @ref ti_sdo_dmai_Smooth module on captured frames since more lines are needed to interpolate the smoothed output.</li>
</ul>
<h4>Defects addressed</h4>
<ul>
   <li>@b SDOCM00046194 DMAI sample applications do not build for DM355 with DVSDK 1.30. <i>Workaround added to release notes.</i></li>
   <li>@b SDOCM00046192 Release notes mention unsupported OMAP3530 build target. <i>Release notes updated.</i></li>
   <li>@b SDOCM00046086 'make help' does not work when building the dmai libraries. <i>Top level Makefile updated to include the help: build target.</i></li>
   <li>@b SDOCM00051496 Venc is not calling XDM_GETBUFINFO to determine codec buffer sizes. <i>Added an XDM_GETBUFINFO call to Venc_create().</i></li>
</ul>
<h4>Compatibility breaks</h4>
N/A

<h3>Since 1.00.00</h3>
<h4>New features</h4>
<ul>
    <li>Introduced support for the DM355 ARM Linux, DM6446 ARM Linux and DM6446,DM6467 and DM6437 DSP DSP/BIOS platforms.</li>
    <li>@c #VideoStd_CIF, @c #VideoStd_SIF_NTSC, @c #VideoStd_SIF_PAL, @c #VideoStd_1080P_30, @c #VideoStd_1080P_25, @c #VideoStd_1080P_24 added.</li>
    <li>@c #Venc1_getReconBufs was added to give access to the reconstructed frames of a video encoder. See the video_encode_io1 application for a dm355 example.</li>
    <li>The @ref ti_sdo_dmai_Loader module now has an async mode (@c #Loader_Attrs.async) allowing you to do the I/O in a separate thread from where you call @c #Loader_getFrame.</li>
    <li>You can now process <a href="user_guide.html#mozTocId46901">partial Graphics Buffers</a>.</li>
    <li>Many new applications were added.</li>
</ul>
<h4>Compatibility breaks</h4>
<ul>
    <li>srcOffset and dstOffset removed from @c #Framecopy_execute, @c #Resize_execute, @c #Ccv_execute, @c #Blend_execute and @c #Smooth_execute. This functionality can instead be achieved with the @c #BufferGfx_Dimensions.x and @c #BufferGfx_Dimensions.y in the @c #BufferGfx_Dimensions, see <a href="user_guide.html#mozTocId46901">the Graphics Buffer Module section</a> in the user guide.</li>
    <li>@c Ccv_convert was moved to simplewidget which is part of the DVSDK demos, and as a consequence @c Ccv_setMode was also removed as it is not needed. The mode is detected from the Buffers you pass to @c #Ccv_config</li>
</ul>
 */
