/*! @page  user_guide Davinci Multimedia Application Interface (DMAI) $version User Guide

<ul class="readonly" id="mozToc">
  <li><a href="#mozTocId650387">1. Introduction: What is the Davinci Multimedia Application Interface (DMAI)?</a></li>
  <li><a href="#mozTocId467901">2. Installing DMAI</a></li>
  <li><a href="#mozTocId467902">3. Using DMAI</a></li>
  <ul>
      <li><a href="#mozTocId467933">3.1 DMAI conventions</a></li>
      <li><a href="#mozTocId467904">3.2 Rebuilding DMAI</a></li>
  </ul>
  <li><a href="#mozTocId46900">4. Module Overview</a></li>
  <ul>
      <li><a href="#mozTocId46901">4.1 Graphics Buffer Module</a></li>
      <li><a href="#mozTocId46951">4.2 Buffer References</a></li>
  </ul>
  <li><a href="#mozTocId467910">5. Debugging a DMAI application</a></li>
  <ul>
      <li><a href="#mozTocId467911">5.1 Enabling DMAI trace</a></li>
      <li><a href="#mozTocId467913">5.2 Buffer information printing</a></li>
  </ul>
  <li><a href="#mozTocId667910">6. DMAI Reference Applications</a></li>
  <ul>
      <li><a href="#mozTocId667911">6.1 Building the applications</a></li>
      <li><a href="#mozTocId667912">6.2 Running the applications on Linux</a></li>
      <li><a href="#mozTocId667913">6.3 Running the applications on DSP/BIOS</a></li>
      <li><a href="#mozTocId667914">6.4 Creating your own application from the reference applications</a></li>
  </ul>
  <li><a href="#mozTocId467903">7. Further reading</a></li>
</ul>

<h2><a class="mozTocH2" name="mozTocId650387"></a>1. Introduction: What is
the Davinci Multimedia Application Interface (DMAI)?</h2>

DMAI is a thin utility layer on top of the operating system (Linux or
DSP/BIOS) and the Codec Engine (CE) to assist in quickly writing
portable applications on a Davinci platform. DMAI is used by the DVSDK demos
and should make additional demo development quicker than in previous DVSDK
releases.

The different DMAI modules communicate using a @ref ti_sdo_dmai_Buffer
abstraction which carries not only the actual data (video, speech, audio etc.)
but also meta data describing the Buffer which is used by the
Codec Engine and Linux device drivers to perform operations on the data.

DMAI does not @e wrap the Operating System or CE, but as the below picture
shows the application can choose when to use DMAI and when to use the
Operating System or Codec Engine directly:

@image html dmai_block.png "DMAI Block Diagram"

DMAI is a functional design, meaning that the modules often describe a
certain operation (frame copying using @ref ti_sdo_dmai_Framecopy,
color conversion using @ref ti_sdo_dmai_Ccv, resizing using @ref
ti_sdo_dmai_Resize etc.), but the module implementation may change between
devices and operating systems depending on which peripheral device drivers and
other local APIs are available.

I.e. DMAI does @b not abstract the peripherals themselves, it abstracts the
actual operations e.g. frame copying. DMAI then implements a frame copy using
the peripherals and resouces at it's disposal on a particular platform (e.g.
the @ref ti_sdo_dmai_Framecopy module is currently implemented using the
resizer peripheral on dm6446, but using DMA on dm355).

DMAI is a collection of modules, and the application can pick and
choose which modules to use. Since DMAI comes with source code, it can
also be used as a reference on how to accomplish certain tasks using
e.g. a certain device driver.

<h2><a class="mozTocH2" name="mozTocId467901"></a>2. Installing DMAI</h2>

If you receive DMAI as part of the DVSDK it will be extracted for you under
<em>\$(DVSDK_INSTALL_DIR)</em>, but if you receive it separate as an update
it will be in a tar ball, i.e. <em>dmai_\<version\>.tar.gz</em>. You extract
the archive using <em>tar xzf dmai_\<version\>.tar.gz</em>.

You can either extract the archive to your <em>\$(DVSDK_INSTALL_DIR)</em>,
since each DMAI release has a unique version number the
installations will not conflict. Or you can extract the archive to another
directory on your system.

DMAI and all of it's sample applications build using the top level
@e Rules.make file, and depending on which of these two above install
options you choose the behavior will be slightly different. This because the
top level @e Rules.make tries to include a @e Rules.make from
<b>one directory above</b> where the DVSDK would keep it's @e Rules.make
if DMAI was delivered as part of a DVSDK, and if found
these settings will be used instead of the local DMAI ones.
This allows the DVSDK
to override how DMAI is built keeping one DVSDK wide <em>Rules.make</em> file,
but also allows DMAI to be built standalone without the DVSDK.

After you have extracted the tar ball, the directory structure will look
as follows:

@verbatim
dmai_<version>                   // Rules.make is in this directory
|-- docs                         // Documentation files for DMAI
`-- packages                     // Your XDC path should include this dir
    `-- ti
        `-- sdo
            `-- dmai             // DMAI package top level directory
                |-- apps         // Reference applications
                |   |
                |   |-- audio_decode1            // audio decode using IAUDDEC1
                |   |-- audio_decode_io          // audio decode using IAUDDEC
                |   |-- audio_decode_io1         // audio decode using IAUDDEC1
                |   |-- audio_encode1            // audio encode using IAUDENC1
                |   |-- audio_encode_io1         // audio encode using IAUDENC1
                |   |-- image_decode_io          // image decode using IIMGDEC
                |   |-- image_decode_io1         // image decode using IIMGDEC1
                |   |-- image_encode_io          // image encode using IIMGENC
                |   |-- image_encode_io1         // image encode using IIMGENC1
                |   |-- speech_decode1           // speech decode using ISPHDEC1
                |   |-- speech_decode_io1        // speech decode using ISPHDEC1
                |   |-- speech_encode1           // speech encode using ISPHENC1
                |   |-- speech_encode_io1        // speech encode using ISPHENC1
                |   |-- video_decode_io          // video decode using IVIDDEC 
                |   |-- video_decode_io2         // video decode using IVIDDEC2
                |   |-- video_display            // video display
                |   |-- video_encode_io          // video encode using IVIDENC 
                |   |-- video_encode_io1         // video encode using IVIDENC1
                |   |-- video_encode_io_multich1 // multi channel video encode using IVIDENC1
                |   |-- video_loopback           // video loopback no copy
                |   |-- video_loopback_blend     // video loopback and blend
                |   |-- video_loopback_convert   // video loopback and color convert
                |   |-- video_loopback_copy      // video loopback copy
                |   `-- video_loopback_resize    // video loopback and resize
                |
                |-- bios          // DSP/BIOS specific source
                |
                |-- ce            // DMAI modules assisting CE usage source
                |
                |-- lib           // Built DMAI libs are put here
                |
                |-- linux         // linux specific source
                |   |-- dm355     // linux dm355 specific source
                |   |-- dm6446    // linux dm6446 / dm357 specific source
                |   |-- dm6467    // linux dm6467 specific source
                |   `-- priv      // Private header files
                |
                |-- obj           // Compiled object files are put here
                `-- priv          // Private header files
@endverbatim

<h2><a class="mozTocH2" name="mozTocId467902"></a>3. Using DMAI</h2>

DMAI depends on the <em>Codec Engine OSAL (Operating System Abstraction Layer)
</em>. In order to use DMAI an application needs to have the following in it's
<em>\<application\>.cfg</em> XDC configuration file
(dm6467 remote linux osal configuration example):

@verbatim
prog.build.platform = "ti.platforms.evmDM6467";

var os = xdc.useModule('ti.sdo.ce.osal.linux.Settings');
osalGlobal.os = os;

var ipc = xdc.useModule('ti.sdo.ce.ipc.Settings');
ipc.commType = ipc.COMM_DSPLINK;

var DMAI = xdc.loadPackage('ti.sdo.dmai');
@endverbatim

See the Codec Engine documentation on how to set up the osal for your platform
configuration. The <em>var DMAI</em> assignment at the bottom is
<b>always the same</b>, the correct library will be linked in automatically.

Each DMAI module is represented with a header file residing in the top level
@e dmai directory. These module interface header files should be referenced
in your app relative to the <em>dmai_\<version\>/packages</em> directory, e.g.
to include the Buffer module you would put the following statement in your
application C source file:

@code
#include <ti/sdo/dmai/Buffer.h>
@endcode

You only need to include the header file of the module you want to use; if a
module depends on other DMAI modules or even device drivers or Codec Engine
headers, it will include them as appropriate.

You should <b>never</b> reference any header files under any of the
<em>priv/</em> directories, as the headers and their contents are considered
internal and may change at any time and your application may break.

<h3><a class="mozTocH3" name="mozTocId467933"></a>3.1 DMAI conventions</h3>

All applications using DMAI modules need to call the @c #Dmai_init function
which is defined in the @ref ti_sdo_dmai_Dmai module @b before using any
other DMAI module API calls.

@note When using Codec Engine and DMAI in the same application, it's currently
necessary to call #Dmai_init @b after calling CERuntime_init.

All modules with an instance object has a @c \<module\>_create API call (e.g.
@c #Display_create) to create this instance object. They also all have a
@c \<module\>_delete function call to delete any resources allocated by
the create call (e.g. @c #Display_delete). The create call returns a handle
with which the instance will be referred to during subsequent module API calls
on this instance. No module APIs can of course be called on an instance handle
before it is created, and no module API:S can be called on an instance handle
after it has been deleted.

All modules with an instance object has an @c \<module\>_Attrs data structure
(e.g. @c #Display_Attrs) which is passed to the @c \<module\>_create call
(e.g. @c #Display_create).
This attributes structure contains configuration parameters for the module
which it uses to create the instance. All @c \<module\>_Attrs structures have
<b>at least</b> one set of defaults (e.g. @c #Display_Attrs_DM6467_VID_DEFAULT)
. Some, like @c #Display_Attrs, have several sets of defaults to accomodate
many typical configurations.

@note Exceptions to this rule are the <em>Codec Engine processing</em>
modules mentioned below (@ref ti_sdo_dmai_Venc, @ref ti_sdo_dmai_Adec etc.)
which only takes the XDM parameters as input to the
@c \<module\>_create calls, but DMAI still provides defaults you should use.

You should @b always set a declared attributes structure to a set of defaults.
This lets the developers add more configuration options in the future without
breaking your application. Typically you would set using the above
mentioned examples:

@code
    Display_Handle hDisplay;
    Display_Attrs dAttrs = Display_Attrs_DM6467_VID_DEFAULT;

    dAttrs.numBufs = 2;

    hDisplay = Display_create(NULL, &dAttrs);
@endcode

The above example will work even if more attributes are added to @c #Display_Attrs, since all values are assigned defaults before a value (@c #Display_Attrs.numBufs) is explicitly modified. By keeping the default behavior the same when adding new configuration parameters, upgrade pains can be kept to a minimum as long as this convention is followed.

To further future proof your application you should always use enum values and defines by their name and not their associated number or value. The value may change with new DMAI versions but the name will not (unless mentioned in the @ref release_notes.

<h3><a class="mozTocH3" name="mozTocId467904"></a>3.2 Rebuilding DMAI</h3>

DMAI comes with full source, partly for debugging purposes and as a reference,
but also so that the user can recompile it with their own modifications.

There are two ways to build DMAI, either inside or outside the DVSDK for a
particular target. Inside means DMAI is extracted inside the DVSDK directory,
@c dvsdk_x_xx_xx_xx, in which case DMAI will pick up the @b DVSDK @e Rules.make
file and import the variables set there. For some targets it may be that DMAI
needs variables set which are not set in the DVSDK Rules.make in which case
they will have to be added. See the @ref release_notes for details for your
particular target.

@note Make sure the PLATFORM variable is set correctly in the top level DVSDK
@e Rules.make file. You can only build DMAI for one platform at a time when
building inside the DVSDK.

@note If the DVSDK already comes with a version of DMAI and you are upgrading,
you will need to change the <tt><b>DMAI_INSTALL_DIR</b></tt> variable in the
DVSDK @e Rules.make file if you want e.g. the DVSDK demos to build using the
upgraded DMAI (see @ref release_notes for compability information for your
target).

To rebuild DMAI extracted outside the DVSDK, you first need to make sure the
top level @b DMAI @e Rules.make points to the correct versions of components
supported. Then from the top level DMAI directory, execute @c @b make to get a
list of available build targets. If you want to compile for e.g. dm6467 ARM
Linux, you would execute <tt><b>make dm6467_al</b></tt>. Building outside the
DVSDK allows building for multiple targets with no change to Rules.make.

@note See the current @ref release_notes for current information on which
component dependencies are required as well as which platforms and build
targets are supported.

DMAI can be built from different levels for a build target. From the top level
the DMAI libraries and all applications will be built. From
@e packages/ti/sdo/dmai only the libraries will be built. From
@e packages/ti/sdo/dmai/apps only the applications will be built, and finally
each application can be rebuilt individually from inside the application top
level directory (e.g. @e audio_decode_io1).

There are two build configurations available, @b debug and @b release. When
debug is selected, no compiler optimizations are performed and assertions are
enabled. This allows you to catch additional errors while developing your
application. For the release configuration assertions are disabled and compiler
optimizations are enabled. You can select the build configuration using the
@c BUILD_TYPE flag (defaults to debug if not set). The same build configurations
are available for the apps. E.g. to build dm6446 ARM Linux for release, execute:

@verbatim
make BUILD_TYPE=release dm6446_al
@endverbatim

In addition, if you want to see how the build invokes the commands, you can
enable verbose printing using the @c VERBOSE flag. E.g. for the example above:

@verbatim
make BUILD_TYPE=release VERBOSE=true dm6446_al
@endverbatim

<h2><a class="mozTocH2" name="mozTocId46900"></a>4. Module Overview</h2>

DMAI consists of several modules which often are independent allowing the user
to select where help is needed. E.g. one may want to handle the display device
driver oneself but leave color conversion to DMAI, and as long as the @ref
ti_sdo_dmai_Buffer module instances are filled in properly this is possible.

There are different types of DMAI modules:

<ol>
    <li> @e Source and @e sink modules (@ref ti_sdo_dmai_Display, @ref
ti_sdo_dmai_Capture, @ref ti_sdo_dmai_Sound and @ref ti_sdo_dmai_Loader)
are used get data in or out of the system using a device driver or a file.</li>
    <li>@e Processing modules (@ref ti_sdo_dmai_Framecopy, @ref ti_sdo_dmai_Ccv,
@ref ti_sdo_dmai_Blend, @ref ti_sdo_dmai_Smooth) are used
to process data in the system.</li>
    <li><em>Codec Engine processing</em> modules (@ref ti_sdo_dmai_Adec, @ref
ti_sdo_dmai_Aenc, @ref ti_sdo_dmai_Adec1, @ref ti_sdo_dmai_Aenc1, @ref
ti_sdo_dmai_Sdec, @ref ti_sdo_dmai_Senc, @ref ti_sdo_dmai_Sdec1, @ref
ti_sdo_dmai_Senc1, @ref ti_sdo_dmai_Ienc, @ref ti_sdo_dmai_Idec, @ref
ti_sdo_dmai_Ienc1, @ref ti_sdo_dmai_Idec1, @ref ti_sdo_dmai_Venc, @ref
ti_sdo_dmai_Venc1, @ref ti_sdo_dmai_Vdec and @ref ti_sdo_dmai_Vdec2)
are used to process data in the system using
the Codec Engine VISA APIs. The VISA handle (e.g. @c VIDDEC_Handle) can
be obtained from the DMAI class allowing custom codec calls to be made
outside the scope of DMAI</li>
    <li><em>Thread communication</em> modules (@ref ti_sdo_dmai_Pause, @ref
ti_sdo_dmai_Rendezvous and @ref ti_sdo_dmai_Fifo) are
used to synchronize and communicate between threads.</li>
    <li><em>Buffer management</em> modules (@ref ti_sdo_dmai_Buffer, @ref
ti_sdo_dmai_BufferGfx and @ref ti_sdo_dmai_BufTab) are important
because they represent the buffer management of DMAI and the Buffer and
BufferGfx modules are the main way with which the other DMAI modules
communicate with each other (and BufTab is a table of Buffer or BufferGfx
instances used by DMAI modules working on full arrays of buffers such as
@ref ti_sdo_dmai_Display, @ref ti_sdo_dmai_Capture and
@ref ti_sdo_dmai_Vdec2).</li>
</ol>

There are also some miscellaneous support modules such as @ref
ti_sdo_dmai_VideoStd, @ref ti_sdo_dmai_ColorSpace, @ref ti_sdo_dmai_Time,
@ref ti_sdo_dmai_Ir, @ref ti_sdo_dmai_Cpu and the
@ref ti_sdo_dmai_Dmai module itself.

The below diagram shows a @e source (@ref ti_sdo_dmai_Loader),
<em>CE processing</em> (@ref ti_sdo_dmai_Vdec) and a @e sink
(@ref ti_sdo_dmai_Display) module, and how they communicate using a
@c #Buffer_Handle:

@image html vdec.png "Video decode"

The below diagram shows a simple audio decode similar to above. Note that the
@c #Sound_write synchronous call is different from the above @c #Display_get
and @c #Display_put asynchronous model:

@image html adec.png "Audio decode"

This diagram shows a more complex scenario where multiple @e processing modules
(@ref ti_sdo_dmai_Ccv, @ref ti_sdo_dmai_Resize) are chained before encoding
the frame (@ref ti_sdo_dmai_Venc) and writing it to a file using stdio:

@image html venc.png "Color conversion, Resizing and Video Encode"

When a @e sink or @e processing module receives an input Buffer it will read
the
current number of bytes used in the Buffer using @c #Buffer_getNumBytesUsed and
process this amount of bytes (as source or destination depending on which
Buffer). This may or may not be the same as the size of the Buffer which can
be retrieved using @c #Buffer_getSize.

When a @e source or @e processing module fills an output Buffer with data it
will
set the number of bytes used in the Buffer using @c #Buffer_setNumBytesUsed to
tell the next module (or the application itself) accepting this Buffer how
many bytes contain valid data.
This of course needs to @b not excess the actual size of the Buffer which can
be retrieved using @c #Buffer_getSize.

<h3><a class="mozTocH3" name="mozTocId46901"></a>4.1 Graphics Buffer Module</h3>

The @ref ti_sdo_dmai_BufferGfx module is a <em>super class</em> of the @ref
ti_sdo_dmai_Buffer module, i.e. in an object oriented language the BufferGfx
module would inherit the Buffer module. This means that each BufferGfx instance
has all the properties of a Buffer instance and @b more; i.e. it has additional
graphics properties (such as @c #BufferGfx_Dimensions and @c #ColorSpace_Type).
This also means that any @ref ti_sdo_dmai_Buffer API calls can @b also be made on any @ref ti_sdo_dmai_BufferGfx module instance, and a BufTab operation also works on each type of Buffer. The base @ref ti_sdo_dmai_Buffer module instances are typically used for file I/O (using @ref ti_sdo_dmai_Loader or stdio) or
@ref ti_sdo_dmai_Sound I/O. Note that both a Buffer and BufferGfx instance are
referenced using a @c #Buffer_Handle.

@image html dimensions.png "Graphics Buffer dimensions"

A BufferGfx instance has dimensions which describe the resolution (with and
height) and the position on the screen as well as the line length (also called
pitch, describes the number of bytes until the next line). These parameters
are used by the DMAI modules to figure out which part of an image to process.

You can use the #BufferGfx_getDimensions call to get the current dimensions and
#BufferGfx_setDimensions to set new dimensions in case you only want to process
part of an image. You normally don't want to change the lineLength parameter.

For example, if you only want to color convert part of an input image you would
call #BufferGfx_setDimensions on the @b input buffer before the call to
#Ccv_execute. Likewise, if your output buffer is larger than the input buffer,
you may want to position the result of the operation inside the output buffer
by calling #BufferGfx_setDimensions on the @b output buffer before calling
#Ccv_execute.

All @ref ti_sdo_dmai_BufferGfx instances get a dimension when they are created,
and these original dimensions cannot be changed. Instead, a subwindow is
recorded when #BufferGfx_setDimensions is called, which will be returned by
subsequent #BufferGfx_getDimensions calls. If you want to restore the original
dimensions you can use the #BufferGfx_resetDimensions call.

It's good practice to always set (or reset) the dimensions of a
graphics buffer before calling a graphics DMAI module function on a buffer. This
way you avoid picking up unwanted dimensions from other processing calls
in your application.

@b Note! Due to the design of XDM not all DMAI modules wrapping Codec Engine
VISA modules set the lineLength correctly, since the frame pitch cannot be
set on a per buffer basis. This can normally be set in the dynamic parameters
of the XDM interface instead, make sure the codec supports this parameter.

<h3><a class="mozTocH3" name="mozTocId46951"></a>4.2 Buffer References</h3>

A feature added in DMAI 1.16 is support for Buffer References. Before when a
@ref ti_sdo_dmai_Buffer instance was created, it always allocated a buffer in
memory for the Buffer instance, but there are cases when this is not desired.

E.g. if a buffer was allocated using a device driver or elsewhere in the system
and this buffer is to be used by DMAI, a physical buffer should not be
allocated. For this and other use cases, the Buffer Reference concept was
introduced.

The #Buffer_Attrs.reference flag indicates whether a buffer is a reference or
not. If set to TRUE, no memory will be allocated for the Buffer instance. In
addition, the API calls #Buffer_setUserPtr and #Buffer_setSize API calls were
added to manipulate a Buffer Reference (note that these API calls do not work
on a non-reference Buffer instance). The #Buffer_isReference call can be used
on a Buffer instance to determine if it's a reference or not.

A Buffer Reference can be used with the DMAI modules just like a non-reference
Buffer instance.

@note For an advanced use case of using a Buffer Reference to implement a ring
buffer, see the @ref ti_sdo_dmai_Loader module.

<h2><a class="mozTocH2" name="mozTocId467910"></a>5. Debugging a DMAI application</h2>

This section discusses some ways of obtaining more information from your
DMAI based application.

<h3><a class="mozTocH3" name="mozTocId467911"></a>5.1 Enabling DMAI trace</h3>

The DMAI modules have trace built in which can be enabled on Linux using the
@c DMAI_DEBUG environment variable. Setting it to '1' prints errors and
warnings, while setting it to '2' prints errors, warnings and debug
messages. If you have a target application @e myapp which uses DMAI you
can set this environment variable for just one invocation of the command
by executing the following on the @b target in the bash shell:

@verbatim
> DMAI_DEBUG=2 ./myapp
@endverbatim

This will send the DMAI trace output to stdout.

<ul>
<li>DMAI_DEBUG=0 disables DMAI trace, even if it is set by Dmai_setLogLevel in the application itself.
<li>DMAI_DEBUG=1: errors and warnings
<li>DMAI_DEBUG=2: all messages 
</ul>

@note CE_DEBUG can be used to independently set the debug level for the Codec Engine. It does not affect the debug level for DMAI.

@note This does @b not require a recompile of DMAI as trace is always built
in.

<h3><a class="mozTocH3" name="mozTocId467913"></a>5.2 Buffer information printing</h3>

There are two DMAI API calls added just to print out information about @ref
ti_sdo_dmai_Buffer instances: @c #Buffer_print is on a per Buffer instance
basis and @c #BufTab_print will print informations for all Buffers in a @ref
ti_sdo_dmai_BufTab.

These functions are quite useful for tracking changes to Buffer state as a
Buffer transitions through a system.

<h2><a class="mozTocH2" name="mozTocId667910"></a>6. DMAI Reference Applications</h2>

The <tt><b>packages/ti/sdo/dmai/apps</b></tt> directory contains reference applications that use DMAI.

The header of each applications's main.c source file contains a brief description of what the application does.

<h3><a class="mozTocH3" name="mozTocId667911"></a>6.1 Building the applications</h3>

Each application uses the XDC configuro tool for configuration of the Codec Engine, codecs, and to load support for DMAI. Loading support for DMAI means querying the DMAI XDC package for the appropriate library that should be linked in by the application to provide the DMAI API.

One or more <tt>*.cfg</tt> files are present in each application directory to describe the corresponding application's configuration. The application Makefile will look for a @c [application]_[device].cfg file, and if it exists the application will be built for this device. For example, the Makefile will find <tt>video_display_dm355.cfg</tt> and determine that this application should be built for the @e dm355 target device.

All applications share the same Makefile in the <tt>packages/ti/sdo/dmai/apps/Makefile.app</tt> file, and common configuration is kept in <tt>packages/ti/sdo/dmai/apps/app_common.cfg</tt>.

An application will try to access the codecs referenced in the @c [application]_[device].cfg file along the paths defined by the @c XDC_PATH environment variable. If these codecs are not found a warning will be issued, but the building process will not fail. Most of the applications use codecs which are part of a DVSDK, however some may try to use codecs which are not included in a DVSDK.

Executing @c make lists the available build targets.

Then, <tt>make [device target]</tt> must be executed to build for a specific target, e.g. <tt>make dm6467_al</tt> to build for ARM Linux on the dm6467 device and <tt>make dm6446_db</tt> to build for DSP/BIOS on the dm6446 device.

Note! If an application fails to build because the configuration step cannot
find the <tt>ti.sdo.dmai</tt> package, make sure you have built DMAI itself for the build target.

<h3><a class="mozTocH3" name="mozTocId667912"></a>6.2 Running the applications on Linux</h3>

Before running the application in a Linux environment, the necessary kernel modules must be loaded using the @c loadmodules.sh script provided with the DVSDK.

The application executable must be loaded to the target. This can be achieved by manual copy or by executing <tt>make install</tt> in the application directory.

If the application is using remote codecs the codec server files (ends with @c .x64P) must be loaded to the same directory as the application executables.

The usage for each application will be provided by invoking the <tt>-h</tt> command-line option. Required parameters must be passed to the application.

Some applications (especially the ones using peripherals) have platform specific
constraints. Read @c [application]/linux/main.c for more information.

DMAI debug trace can be enabled by setting the DMAI_DEBUG environment variable
in Linux. Setting it to 1 corresponds to errors and warnings only, 2 corresponds
to all trace (see <a href="#mozTocId467911">5.1 Enabling DMAI trace</a>). 

<h3><a class="mozTocH3" name="mozTocId667913"></a>6.3 Running the applications on DSP/BIOS</h3>

In order to run the DSP/BIOS applications, CCS 3.3 and a JTAG emulator are required.

In order to build the application for a specific platform, the [application]/bios directory must include a [application]_[device].cfg and a [application]_[device].tcf. For example, in order to run the audio_decode_io1 DSP/BIOS application on the dm6467 platform, the audio_decode_io1/bios directory must include the files audio_decode_io1_dm6467.cfg and audio_decode_io1_dm6467.tcf. If these files are not included, these files can be created by copying and renaming files existing for a different platform. After re-naming the files, the .tcf file does not require any changes but the .cfg file must be updated to reflect the codec package used. The names used for the engine and the codec must be used as arguments in the args.c file.

When using CCS, each platform requires specific GEL files which set up the memory map. CCS 3.3.81.5 includes the necessary GEL files. If the CCS used does not include the GEL files, download them from <a href="http://support.spectrumdigital.com/index.html?osCsid=53e727c80ba4c88096dd0f79f9a1c780">Spectrum Digital</a>.

In order to connect the board to CCS an emulator is required. The emulator drivers must be installed before connecting the board to CCS.

Before using CCS, launch CCS Setup and select the Factory Board required by the platform and the emulator used. Right-click on the Processor and select Properties. If the GEL file is not associated with that processor, load the GEL file. After completing the Setup, launch CCS. On heterogenous platforms with ARM and DSP, you need to bring the DSP out of reset before you can connect to it via CCS: 

<b>DM6467 EVM</b>

Before powering ON the board set SW3 Boot Mode Configuration switch to the No Boot position: SW3[4:1]=1111. 
<ol>
<li> In the Parallel Debug Manager, right-click on the ICEPICK and click Connect Device</li>
<li> Right-click on the ARM926 and select Open. This will open a CCS control window to the ARM. Connect to the ARM by selecting Debug->Connect from the menu.</li>
<li> Select GEL->Davinci HD DSP->DSP_Boot_from_DDR2 to release the DSP from reset.</li>
<li> Disconnect from the ARM by hitting Alt-C (Debug Disconnect)</li>
<li> Open a control window to the DSP from the Parallel Debug Manager by right clicking on C6400PLUS and selecting Open.</li>
<li> Connect to DSP - Alt-C (Debug Connect)</li>
<li> In CCS, after connecting to the DSP, do the following:
  <ol>
  <li>Select File->Load GEL to open the GEL file dmai.gel in the apps/ directory.</li>
  <li>Load the executable of your choice.</li>
  <li>Select Debug->Go Main to go to main().</li>
  <li>Select GEL->Dmai->Setup_Rtdx_Cio to setup CIO to use RTDX if you intend to use RTDX. (see below for more information about RTDX).</li>
  <li>Optionally select GEL->Dmai->Set_debugLogLevel_1 or GEL->Dmai->Set_debugLogLevel_2 to set the debug level of your choice, if you want debug trace to appear on stdout. 1 corresponds to errors and warnings only, 2 corresponds to all trace. Note that you MUST do this prior to the code running Dmai_init().</li>
  <li>Hit F5 to run the application. Note that in DSP/BIOS, the application does not automatically halt when it reaches the end.</li>
  </ol>
</ol>

<b>DM6446 EVM</b>
 
Before powering ON the board set S3 Configuration switch to 0011111110. 
The steps for the DM6446 EVM are similar to the steps described above for DM6467 EVM: Connect to ARM, Release the DSP, Connect to DSP. 

<b>General</b>

If you are running video applications that do file I/O (ie. all applications with names that end with @e _io), it is strongly recommended to download and install
RTDX 2.10 or above from <a href="https://www-a.ti.com/downloads/sds_support/targetcontent/rtdx/index.html">here</a>.

This enhances CIO by making it up to 60 times faster. After installing it, you
should perform the following:

<ol>
<li>You may need to replace RtdxConfig.ocx in CCS 3.3 (@e C:/CCSTUDIO_V3.3/plugins/rtdx) with the one provided in the RTDX installation. Make sure you move the original file outside of its folder, otherwise CCS will still be able to find it despite you renaming it.</li>
<li>In CCSetup, change the emulator TCLK speed by right-clicking on
your emulator on the System Configuration pane, choose properties, and change
JTAG TCLK Frequency to "Automatic with legacy 10.368MHz limit"</li>
<li>Enable CIO to use RTDX by uncommenting sections of the code that does this in the .cfg and .tcf files of the application under the @b [application]/bios/ directory. Then rebuild the application.</li>
</ol>

In addition to the DMAI debug trace, it is also possible to enable in CCS the Codec Engine debug trace. See more details here <a href="http://wiki.davincidsp.com/index.php?title=Codec_Engine_FAQ">Codec Engine FAQ</a>.

<h3><a class="mozTocH3" name="mozTocId667914"></a>6.4 Creating your own application from the reference applications</h3>

If you want to create your own application using these applications as a reference, and want to put them in your own directory outside the DMAI directory structure, make sure the @c DMAI_INSTALL_DIR variable is set correctly at the top of your Makefile. The DMAI application Makefile uses the top level @c Rules.make to find the components it needs to build the application, and setting the @c DMAI_INSTALL_DIR variable allows the Makefile to find this file. 

<h2><a class="mozTocH2" name="mozTocId467903"></a>7. Further reading</h2>

<ul>
    <li>Codec Engine Application Developer User's Guide (@e sprue67d).
</ul>
 */
