# --COPYRIGHT--,BSD
#  Copyright (c) $(CPYYEAR), Texas Instruments Incorporated
#  All rights reserved.
# 
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
#  are met:
# 
#  *  Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
# 
#  *  Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
# 
#  *  Neither the name of Texas Instruments Incorporated nor the names of
#     its contributors may be used to endorse or promote products derived
#     from this software without specific prior written permission.
# 
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
#  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
#  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
#  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
#  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
#  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
#  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
#  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
#  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# --/COPYRIGHT--
#!/bin/sh

check_status() {
    ret=$?
    if [ "$ret" -ne "0" ]
    then
        echo "Failed making release, aborting.."
        exit
    fi
}

echo "*************************************************************************"
echo "* Making release of DMAI version $1"
echo "*"
echo "* This script needs to run on avocado.sanb.design.ti.com!"
echo "*"
echo "* Make sure you have edited the top level Rules.make to fit your system."
echo "*************************************************************************"
echo
echo "Press return to continue.."

read x

#!/bin/sh

date=`date`
ystring=`echo "$date" | sed "s/.* \([0-9][0-9][0-9][0-9]\)$/\1/g"`

echo
echo "Replacing \$(CPYYEAR) with $ystring in source files..."
echo

c_files=`find . -type f -name "*.c"`
h_files=`find . -type f -name "*.h"`
xs_files=`find . -type f -name "*.xs"`
xdc_files=`find . -type f -name "*.xdc"`
bld_files=`find . -type f -name "*.bld"`
cfg_files=`find . -type f -name "*.cfg"`
Makefiles=`find . -type f -name "Makefile*"`

files="${c_files} ${h_files} ${xs_files} ${xdc_files} ${bld_files} ${cfg_files} ${Makefiles} dmai.mpi"

for file in $files
do
    sed "s/\$(CPYYEAR)/$ystring/g" < $file > $file.new
    check_status

    mv $file.new $file
    check_status
done

vstring=`echo "$1" | sed "s/\([0-9][0-9]*\)_\([0-9][0-9]\)_\([0-9][0-9]\)_\([0-9][0-9]\)/\1.\2.\3 Build \4/g"`

files="docs/main_page.h docs/release_notes.h docs/user_guide.h docs/license.h"

for file in $files
do
    sed "s/\$version/$vstring/g" < $file > $file.new
    check_status

    mv $file.new $file
    check_status
done

echo
echo "Creating docs for DMAI..."
echo

tdox "docs dmai/packages/ti/sdo/dmai" dmai/docs -n "Davinci Multimedia Application Interface (DMAI)" -m "../dmai_$1.chm" -v $1 -f *.h -o docs/doxyfile_override -e "*priv/* *apps/*"

check_status

echo
echo "Creating documentation links..."
echo

echo '<meta http-equiv="refresh" content="0; url=docs/html/release_notes.html">' > "dmai/dmai_$1_ReleaseNotes.html"

check_status

echo '<meta http-equiv="refresh" content="0; url=docs/html/user_guide.html">' > "dmai/dmai_$1_UserGuide.html"

check_status

echo '<meta http-equiv="refresh" content="0; url=docs/html/modules.html">' > "dmai/dmai_$1_ApiReference.html"

check_status

echo '<meta http-equiv="refresh" content="0; url=docs/html/license.html">' > "dmai/dmai_$1_License.html"

echo
echo "Building DMAI..."
echo

if [ -f make.dm357_al ]
then
    ./make.dm357_al -C dmai dm357_al
    check_status
fi

if [ -f make.o3530_al ]
then
    ./make.o3530_al -C dmai o3530_al
    check_status
fi

if [ -f make.dm6446_al ]
then
    ./make.dm6446_al -C dmai dm6446_al
    check_status
fi

if [ -f make.dm6467_al ]
then
    ./make.dm6467_al -C dmai dm6467_al
    check_status
fi

if [ -f make.dm355_al ]
then
    ./make.dm355_al -C dmai dm355_al
    check_status
fi

if [ -f make.dm6446_db ]
then
    ./make.dm6446_db -C dmai dm6446_db
    check_status
fi

if [ -f make.dm6467_db ]
then
    ./make.dm6467_db -C dmai dm6467_db
    check_status
fi

if [ -f make.dm6437_db ]
then
    ./make.dm6437_db -C dmai dm6437_db
    check_status
fi

echo
echo "Creating release dmai_$1.tar.gz"
echo

mv dmai dmai_$1
check_status

ln -s dmai_$1 dmai
check_status

tar czf dmai_$1.tar.gz dmai_$1
check_status

sed "s/\__Version__/$1/g" < dmai.mpi > dmai.mpi.new
check_status

mv dmai.mpi.new dmai.mpi
check_status

/db/rtree/library/vendors/installjammer/1.2.7/installjammer/installjammer --build --build-dir /tmp --output-dir . -DVersion $1 -- dmai.mpi
check_status

echo
echo "Copying release to /db/public/home/SDOApps/dmai/dmai_$1"
echo

mkdir /db/public/home/SDOApps/dmai/dmai_$1
check_status

chmod 755 /db/public/home/SDOApps/dmai/dmai_$1
check_status

cp dmai_$1.tar.gz /db/public/home/SDOApps/dmai/dmai_$1
check_status

chmod 644 /db/public/home/SDOApps/dmai/dmai_$1/dmai_$1.tar.gz
check_status

cp dmai_setuplinux_$1.bin /db/public/home/SDOApps/dmai/dmai_$1
check_status

chmod 644 /db/public/home/SDOApps/dmai/dmai_$1/dmai_setuplinux_$1.bin
check_status

cp dmai_setupwin32_$1.exe /db/public/home/SDOApps/dmai/dmai_$1
check_status

chmod 644 /db/public/home/SDOApps/dmai/dmai_$1/dmai_setupwin32_$1.exe
check_status

cp dmai_$1/docs/dmai_$1.chm /db/public/home/SDOApps/dmai/dmai_$1
check_status

chmod 644 /db/public/home/SDOApps/dmai/dmai_$1/dmai_$1.chm
check_status

echo
echo "Done making dmai_$1 release"
echo
