# --COPYRIGHT--,BSD
#  Copyright (c) $(CPYYEAR), Texas Instruments Incorporated
#  All rights reserved.
# 
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions
#  are met:
# 
#  *  Redistributions of source code must retain the above copyright
#     notice, this list of conditions and the following disclaimer.
# 
#  *  Redistributions in binary form must reproduce the above copyright
#     notice, this list of conditions and the following disclaimer in the
#     documentation and/or other materials provided with the distribution.
# 
#  *  Neither the name of Texas Instruments Incorporated nor the names of
#     its contributors may be used to endorse or promote products derived
#     from this software without specific prior written permission.
# 
#  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
#  THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#  PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
#  CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
#  EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
#  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
#  OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
#  WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
#  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
#  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
# --/COPYRIGHT--
#!/usr/bin/perl

###############################################################################
# Calculate device memory settings
###############################################################################
my $phys_mem_base    = hex("0x80000000");
my $total_device_mem = 128 * 1024 * 1024;
my $avail_device_mem = 0;

# Calculate amount of device memory not in use by Linux
if (open(CMDLINE, "/proc/cmdline")) {

    while (chomp($_ = <CMDLINE>)) {
        if (/mem=(\d+)M/) {
            $avail_device_mem = $total_device_mem - ($1 * 1024 * 1024);
            last;
        }
    }
}
close(CMDLINE);
printf("# Available device memory: %uMB\n", $avail_device_mem / 1024 / 1024);

###############################################################################
# Read current CMEM settings
###############################################################################
my $cmem_setting = "";

if (open(CMEM, "/proc/cmem")) {

    my @pools;

    while (chomp($_ = <CMEM>)) {
        if (/Pool \d+: (\d+) bufs size \d+ \((\d+) requested\)/) {
            push @pools, "$1x$2";
        }
    }
    $cmem_setting = join(",", @pools) . "\n";
}
close(CMEM);

###############################################################################
# Calculate CMEM start and end addresses
###############################################################################
my $phys_start = $phys_mem_base + $total_device_mem - $avail_device_mem;
my $phys_end   = $phys_mem_base + $total_device_mem;

###############################################################################
# Execute commands to configure CMEM, run the app, and put CMEM back to its
# original settings.
###############################################################################
printf("insmod cmemk.ko phys_start=0x%08x phys_end=0x%08x " .
       "pools=%s\n", $phys_start, $phys_end, $cmem_setting);
