/* --COPYRIGHT--,BSD
 * Copyright (c) $(CPYYEAR), Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

#include <stdio.h>

#include <xdc/std.h>

#include "appMain.h"

#include <ti/sdo/dmai/Cpu.h>

extern Void sleepForOneSecond();

/******************************************************************************
 * appMain
 ******************************************************************************/
Void appMain()
{
    Cpu_Attrs  cpuAttrs = Cpu_Attrs_DEFAULT;
    Cpu_Handle hCpu;
    Cpu_Device device;
    Int        load;
    Int        i;

    printf("CPU Test started\n");

    /* Initialize DMAI */
    Dmai_init();
    
    hCpu = Cpu_create(&cpuAttrs);

    if (hCpu == NULL) {
        printf("Failed to create CPU object\n");
        return;
    }

    if (Cpu_getDevice(hCpu, &device) < 0) {
        printf("Failed to get CPU type\n");
        goto cleanup;
    }

    for (i = 0; i < 10; i++) {
        if (Cpu_getLoad(hCpu, &load) < 0) {
            printf("Failed to get CPU load\n");
            goto cleanup;
        }

        printf("Cpu is of type %s and is loaded %d%%\n",
               Cpu_getDeviceName(device), load);

        sleepForOneSecond();
    }

cleanup:
    Cpu_delete(hCpu);

    return;
}
