/* --COPYRIGHT--,BSD
 * Copyright (c) $(CPYYEAR), Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>

#include <xdc/std.h>

#include "../appMain.h"

/* Default arguments for app */
#define DEFAULT_ARGS { 640, 480, 1000, FALSE}

/*
 * Argument IDs for long options. They must not conflict with ASCII values,
 * so start them at 256.
 */
typedef enum
{
   ArgID_RES = 256,
   ArgID_NUMFRAMES,
   ArgID_BENCHMARK,
   ArgID_HELP,
   ArgID_NUMARGS
} ArgID;

/******************************************************************************
 * usage
 ******************************************************************************/
static Void usage(void)
{
    fprintf (stderr, "Usage framecopy_<platform> [options]\n\n"
     "Options:\n"
     "       --benchmark    Print benchmarking infomration\n"
     " -r  | --resolution   Input resolution [Default:640x480]\n"
     " -n  | --numframes    Number of frames [Default:100]\n"
     " -h | --help          Print this help information\n"
     );
}

/******************************************************************************
 * parseArgs
 *****************************************************************************/
static Void parseArgs(Int argc, Char *argv[], Args *argsp)
{
    const char shortOptions[] = "r:n:h";

    const struct option longOptions[] = {
        {"resolution",   required_argument,  NULL, ArgID_RES},
        {"numframes",   required_argument,  NULL, ArgID_NUMFRAMES},
        {"benchmark", no_argument,  NULL, ArgID_BENCHMARK},
        {"help" ,   no_argument, NULL, ArgID_HELP},
        {0,0,0,0}
    };

    Int index;
    Int argID;

    for (;;) {
        argID = getopt_long(argc, argv, shortOptions, longOptions, &index);

        if (argID == -1) {
            break;
        }

        switch (argID) {

             case ArgID_RES:
             case 'r':
                    if (sscanf(optarg,"%dx%d", &argsp->width, &argsp->height) 
                            !=2 )
                    {
                        fprintf (stderr, "Invalid resolution!\n");
                        usage ();
                        exit (EXIT_FAILURE);
                    }   
                    break;

             case ArgID_NUMFRAMES:
             case 'n':
                    argsp->numFrames = atoi (optarg);
                    break;

             case ArgID_HELP:
             case 'h':
                    usage ();
                    exit (EXIT_SUCCESS);
                    break;

             case ArgID_BENCHMARK:
                    argsp->benchmark = TRUE;
                    break;

             default:
                    usage ();
                    exit (EXIT_FAILURE);
        }
    } 

    if (optind < argc) {
         usage ();
         exit (EXIT_FAILURE);
    }
}


/******************************************************************************
 * main
 *****************************************************************************/
Int main (Int argc, Char *argv[])
{
    Args args = DEFAULT_ARGS;

    /* Parse the argument given to the app */
    parseArgs(argc, argv, &args);

    return appMain(&args);
}

