/* --COPYRIGHT--,BSD
 * Copyright (c) $(CPYYEAR), Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <xdc/std.h>

#include <ti/sdo/dmai/Ir.h>

#include "appMain.h"

extern Void sleepForOneSecond();

/******************************************************************************
 * appMain
 ******************************************************************************/
Void appMain()
{
    Ir_Attrs   irAttrs  = Ir_Attrs_DEFAULT;
    Ir_Handle  hIr      = NULL;
    Ir_Key key;

    printf("IR Test started\n");

    /* Initialize DMAI */
    Dmai_init();

    hIr = Ir_create(&irAttrs);

    if (hIr == NULL) {
        printf("Failed to initialize IR\n");
        goto cleanup;
    }

    while (1) {
        if (Ir_getKey(hIr, &key) < 0) {
            printf("Failed to get IR key\n");
            goto cleanup;
        }

        switch (key) {
            case 0:
                printf("No key pressed\n");
                break;
            case Ir_Key_POWER:
                printf("Power pressed\n");
                break;
            case Ir_Key_CHANINC:
                printf("Chaninc pressed\n");
                break;
            case Ir_Key_CHANDEC:
                printf("Chandec pressed\n");
                break;
            case Ir_Key_VOLINC:
                printf("Volinc pressed\n");
                break;
            case Ir_Key_VOLDEC:
                printf("Voldec pressed\n");
                break;
            case Ir_Key_OK:
                printf("Ok pressed\n");
                break;
            case Ir_Key_MENU:
                printf("Menu pressed\n");
                break;
            case Ir_Key_MENUDONE:
//            case Ir_Key_INFOSELECT:
                printf("Menudone or infoselect pressed\n");
                break;
            case Ir_Key_SLEEP:
                printf("Sleep pressed\n");
                break;
            case Ir_Key_SUBTITLE:
                printf("Subtitle pressed\n");
                break;
            case Ir_Key_REPEAT:
                printf("Repeat pressed\n");
                break;
            case Ir_Key_1:
                printf("1 pressed\n");
                break;
            case Ir_Key_2:
                printf("2 pressed\n");
                break;
            case Ir_Key_3:
                printf("3 pressed\n");
                break;
            case Ir_Key_4:
                printf("4 pressed\n");
                break;
            case Ir_Key_5:
                printf("5 pressed\n");
                break;
            case Ir_Key_6:
                printf("6 pressed\n");
                break;
            case Ir_Key_7:
                printf("7 pressed\n");
                break;
            case Ir_Key_8:
                printf("8 pressed\n");
                break;
            case Ir_Key_9:
                printf("9 pressed\n");
                break;
            case Ir_Key_0:
                printf("0 pressed\n");
                break;
            case Ir_Key_ENTER:
                printf("Enter pressed\n");
                break;
            case Ir_Key_INPUT:
                printf("Input pressed\n");
                break;
            case Ir_Key_REWIND:
                printf("Rewind pressed\n");
                break;
            case Ir_Key_PLAY:
                printf("Play pressed\n");
                break;
            case Ir_Key_FASTFORWARD:
                printf("Fastforward pressed\n");
                break;
            case Ir_Key_RECORD:
                printf("Record pressed\n");
                break;
            case Ir_Key_STOP:
                printf("Stop pressed\n");
                break;
            case Ir_Key_PAUSE:
                printf("Pause pressed\n");
                break;
            default:
                printf("Unknown key pressed (0x%x)\n", key);
                break;
        }

        sleepForOneSecond();
    }

cleanup:
    if (hIr) {
        Ir_delete(hIr);
    }

    return;
}
