/* --COPYRIGHT--,BSD
 * Copyright (c) $(CPYYEAR), Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

#include <stdio.h>
#include <string.h>

#include <xdc/std.h>

#include <ti/sdo/dmai/Sound.h>
#include <ti/sdo/dmai/BufTab.h>

#include "appMain.h"

#define NUM_BUFS 1

/******************************************************************************
 * appMain
 ******************************************************************************/
Void appMain(Args *args)
{
    Buffer_Attrs     bAttrs       = Buffer_Attrs_DEFAULT;
    Sound_Attrs      sAttrs       = Sound_Attrs_STEREO_DEFAULT;
    Sound_Handle     hSound       = NULL;
    BufTab_Handle    hBufTab      = NULL;
    Buffer_Handle    hInBuf;
    Int              bufSize;

    /* Initialize DMAI */
    Dmai_init();

    if (args->channels != -1) {
        sAttrs.channels = args->channels;
    }

    if (args->sampleRate != -1) {
        sAttrs.sampleRate = args->sampleRate;
    }

    if (args->soundStd != -1) {
        sAttrs.soundStd = args->soundStd;
    }

    if (args->bufSize != -1) {
        sAttrs.bufSize = args->bufSize;
    }

    bufSize = sAttrs.bufSize;
    hBufTab = BufTab_create(NUM_BUFS, bufSize, &bAttrs);

    if (hBufTab == NULL) {
        printf("Failed to allocate mem tab\n");
        goto cleanup;
    }

    hSound = Sound_create(&sAttrs);

    if (hSound == NULL) {
        printf("Failed to create audio device\n");
        goto cleanup;
    }

    hInBuf = BufTab_getFreeBuf(hBufTab);

    if (hInBuf == NULL) {
        printf("Failed to get free contiguous buffers\n");
        goto cleanup;
    }

    while (args->numFrames > 0) {
        if (Sound_read(hSound, hInBuf) < 0) {
            printf("Failed to read audio buffer\n");
            goto cleanup;
        }

        if (Sound_write(hSound, hInBuf) < 0) {
            printf("Failed to write audio buffer\n");
            goto cleanup;
        }

        args->numFrames--;
    }

cleanup:
    if (hBufTab) {
        BufTab_delete(hBufTab);
    }

    if (hSound) {
        Sound_delete(hSound);
    }

    return;
}
