/*
 * main.c
 *
 * ============================================================================
 * Copyright (c) Texas Instruments Inc 2007
 *
 * Use of this software is controlled by the terms and conditions found in the
 * license agreement under which this software has been supplied or provided.
 * ============================================================================
 */

#include <stdio.h>
#include <string.h>

#include <xdc/std.h>

#include <ti/sdo/dmai/Sound.h>
#include <ti/sdo/dmai/BufTab.h>

#include "appMain.h"

/******************************************************************************
 * appMain
 ******************************************************************************/
Void appMain(Args * args)
{
    Buffer_Attrs     bAttrs       = Buffer_Attrs_DEFAULT;
    Sound_Attrs      sAttrs       = Sound_Attrs_STEREO_DEFAULT;
    Sound_Handle     hSound       = NULL;
    Buffer_Handle    hInBuf       = NULL;
    FILE            *fp           = NULL;
    Int              numFrame     = 0;
    Int              numBytes;
    Int              bufSize;

    /* Initialize DMAI */
    Dmai_init();

    fp = fopen(args->outFile, "wb");

    if (fp == NULL) {
        printf("Failed to open output file %s\n", args->outFile);
        goto cleanup;
    }

    sAttrs.mode = Sound_Mode_INPUT;

    if (args->channels != -1) {
        sAttrs.channels = args->channels;
    }

    if (args->sampleRate != -1) {
        sAttrs.sampleRate = args->sampleRate;
    }

    if (args->soundStd != -1) {
        sAttrs.soundStd = args->soundStd;
    }

    if (args->bufSize != -1) {
        sAttrs.bufSize = args->bufSize;
    }

    hSound = Sound_create(&sAttrs);

    if (hSound == NULL) {
        printf("Failed to create audio device\n");
        goto cleanup;
    }

    bufSize = sAttrs.bufSize;
    hInBuf = Buffer_create(bufSize, &bAttrs);

    if (hInBuf == NULL) {
        printf("Failed to create buffer\n");
        goto cleanup;
    }

    while (numFrame++ < args->numFrames) {
        if (Sound_read(hSound, hInBuf) < 0) {
            printf("Failed to write audio buffer\n");
            goto cleanup;
        }

        numBytes = fwrite(Buffer_getUserPtr(hInBuf), 1,
                          Buffer_getNumBytesUsed(hInBuf), fp);

        if (numBytes != Buffer_getNumBytesUsed(hInBuf)) {
            printf("Failed to write to file %s\n", args->outFile);
            goto cleanup;
        }

        printf("Frame %d size %d written to file\n", numFrame, numBytes);

    }

cleanup:
    if (hInBuf) {
        Buffer_delete(hInBuf);
    }

    if (hSound) {
        Sound_delete(hSound);
    }

    if (fp) {
        fclose(fp);
    }

    return;
}
