/* --COPYRIGHT--,BSD
 * Copyright (c) $(CPYYEAR), Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <string.h>

#include <xdc/std.h>

#include "../appMain.h"

/* Default arguments for app */
#define DEFAULT_ARGS { 100, -1, -1, -1, -1 }

/*
 * Argument IDs for long options. They must not conflict with ASCII values,
 * so start them at 256.
 */
typedef enum
{
   ArgID_HELP = 256,
   ArgID_OUTPUT_FILE,
   ArgID_NUMFRAMES,
   ArgID_CHANNELS,
   ArgID_RATE,
   ArgID_SOUND_STD,
   ArgID_BUFFER_SIZE,
   ArgID_NUMARGS
} ArgID;

/******************************************************************************
 * usage
 ******************************************************************************/
static Void usage(void)
{
    fprintf(stderr, "Usage: sound_read_<platform> [options]\n\n"
        "Options:\n"
        "     --sample_rate  Sampling rate \n"
        "     --buffer_size  Write buffer size \n"
        "     --channels     Number of channel \n"
        "     --sound_std    Sound Standard to use (see below)\n"
        "-h | --help         Print usage information (this message)\n"
        "-o | --output_file  Name of input file to decode\n"
        "-n | --numframes    Number of frames to process [Default: 100]\n\n");
}

/******************************************************************************
 * parseArgs
 ******************************************************************************/
static Void parseArgs(Int argc, Char *argv[], Args *argsp)
{
    const char shortOptions[] = "ho:n:";
    const struct option longOptions[] = {
        {"help",        no_argument,       NULL, ArgID_HELP        },
        {"output_file", required_argument, NULL, ArgID_OUTPUT_FILE },
        {"numframes",   required_argument, NULL, ArgID_NUMFRAMES   },
        {"channels",    required_argument, NULL, ArgID_CHANNELS    },
        {"sample_rate", required_argument, NULL, ArgID_RATE        },
        {"buffer_size", required_argument, NULL, ArgID_BUFFER_SIZE },
        {"sound_std",   required_argument, NULL, ArgID_SOUND_STD   },
        {0, 0, 0, 0}
    };
    Int     outfile = FALSE;
    Int     index;
    Int     argID;

    for (;;) {
        argID = getopt_long(argc, argv, shortOptions, longOptions, &index);

        if (argID == -1) {
            break;
        }

        switch (argID) {

            case ArgID_HELP:
            case 'h':
                usage();
                exit(EXIT_SUCCESS);

            case ArgID_OUTPUT_FILE:
            case 'o':
                strncpy(argsp->outFile, optarg, MAX_FILE_NAME_SIZE);
                outfile = TRUE;
                break;

            case ArgID_NUMFRAMES:
            case 'n':
                argsp->numFrames = atoi(optarg);
                break;

            case ArgID_CHANNELS:
                argsp->channels = atoi(optarg);
                break;

            case ArgID_RATE:
                argsp->sampleRate = atoi(optarg);
                break;

            case ArgID_BUFFER_SIZE:
                argsp->bufSize = atoi(optarg);
                break;

            case ArgID_SOUND_STD:
                switch(atoi(optarg)) {
                    case 1:
                        argsp->soundStd = Sound_Std_OSS;
                        break;
                    case 2:
                        argsp->soundStd = Sound_Std_ALSA;
                        break;
                    default:
                        fprintf(stderr, "Unknown sound standard\n");
                        usage();
                        exit(EXIT_FAILURE);
                }
                break;


            default:
                usage();
                exit(EXIT_FAILURE);
        }
    }

    if (optind < argc) {
        usage();
        exit(EXIT_FAILURE);
    }
}

/******************************************************************************
 * main
 ******************************************************************************/
Int main(Int argc, Char *argv[])
{
    Args args = DEFAULT_ARGS;
    
    /* Parse the arguments given to the app */
    parseArgs(argc, argv, &args);    

    appMain(&args);

    return 0;
}
