/*
 * main.c
 *
 * ============================================================================
 * Copyright (c) Texas Instruments Inc 2007
 *
 * Use of this software is controlled by the terms and conditions found in the
 * license agreement under which this software has been supplied or provided.
 * ============================================================================
 */

#include <stdio.h>
#include <string.h>

#include <xdc/std.h>

#include <ti/sdo/dmai/Sound.h>
#include <ti/sdo/dmai/BufTab.h>

#include "appMain.h"

/******************************************************************************
 * appMain
 ******************************************************************************/
Void appMain(Args * args)
{
    Buffer_Attrs     bAttrs       = Buffer_Attrs_DEFAULT;
    Sound_Attrs      sAttrs       = Sound_Attrs_STEREO_DEFAULT;
    Sound_Handle     hSound       = NULL;
    Buffer_Handle    hOutBuf      = NULL;
    FILE            *fp           = NULL;
    Int              numFrame     = 0;
    Int              numBytes;
    Int              bufSize;

    /* Initialize DMAI */
    Dmai_init();

    fp = fopen(args->inFile, "rb");

    if (fp == NULL) {
        printf("Failed to open output file\n");
        goto cleanup;
    }

    sAttrs.mode = Sound_Mode_OUTPUT;

    if (args->channels != -1) {
        sAttrs.channels = args->channels;
    }

    if (args->sampleRate != -1) {
        sAttrs.sampleRate = args->sampleRate;
    }

    if (args->soundStd != -1) {
        sAttrs.soundStd = args->soundStd;
    }

    if (args->bufSize != -1) {
        sAttrs.bufSize = args->bufSize;
    }

    hSound = Sound_create(&sAttrs);

    if (hSound == NULL) {
        printf("Failed to create audio device\n");
        goto cleanup;
    }

    bufSize = sAttrs.bufSize;

    hOutBuf = Buffer_create(bufSize, &bAttrs);

    if (hOutBuf == NULL) {
        printf("Failed to create buffer\n");
        goto cleanup;
    }

    while (numFrame++ < args->numFrames) {
        numBytes = fread(Buffer_getUserPtr(hOutBuf), 1, bufSize, fp);

        if (numBytes != bufSize) {
            if (ferror(fp)) {
                printf("Failed to read from file %s\n", args->inFile);
                goto cleanup;
            }
        }

        Buffer_setNumBytesUsed(hOutBuf, numBytes);

        if (Sound_write(hSound, hOutBuf) < 0) {
            printf("Failed to write audio buffer\n");
            goto cleanup;
        }

        if (feof(fp)) {
            printf("End of file reached, exiting\n");
            break;
        }

//        printf("Frame %d size %d bytes written to device driver\n",
//               numFrame, numBytes);
    }

cleanup:
    if (hOutBuf) {
        Buffer_delete(hOutBuf);
    }

    if (hSound) {
        Sound_delete(hSound);
    }

    if (fp) {
        fclose(fp);
    }

    return;
}
