/* --COPYRIGHT--,BSD
 * Copyright (c) $(CPYYEAR), Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

#include <stdio.h>
#include <string.h>

#include <xdc/std.h>

#include <ti/sdo/ce/Engine.h>
#include <ti/sdo/ce/CERuntime.h>

#include <ti/sdo/dmai/Dmai.h>
#include <ti/sdo/dmai/Sound.h>
#include <ti/sdo/dmai/Loader.h>
#include <ti/sdo/dmai/BufTab.h>
#include <ti/sdo/dmai/ce/Sdec.h>

#include "appMain.h"

#define BUFSIZE 160
#define NUM_BUFS 2

/******************************************************************************
 * appMain
 ******************************************************************************/
Void appMain(Args * args)
{
    Sound_Attrs             sAttrs      = Sound_Attrs_MONO_DEFAULT;
    Loader_Attrs            lAttrs      = Loader_Attrs_DEFAULT;
    Buffer_Attrs            bAttrs      = Buffer_Attrs_DEFAULT;
    SPHDEC_Params           params      = Sdec_Params_DEFAULT;
    SPHDEC_DynamicParams    dynParams   = Sdec_DynamicParams_DEFAULT;
    Sdec_Handle             hSd         = NULL;
    Sound_Handle            hSound      = NULL;
    Loader_Handle           hLoader     = NULL;
    Engine_Handle           hEngine     = NULL;
    Buffer_Handle           hOutBuf     = NULL;
    Buffer_Handle           hInBuf;


    /* Initialize the codec engine run time */
    CERuntime_init();

    /* Initialize DMAI */
    Dmai_init();
    
    hSound = Sound_create(&sAttrs);

    if (hSound == NULL) {
        printf("Failed to create audio device\n");
        goto cleanup;
    }

    /* Open the codec engine */
    hEngine = Engine_open(args->engineName, NULL, NULL);

    if (hEngine == NULL) {
        printf("Failed to open codec engine %s\n", args->engineName);
        goto cleanup;
    }

    /* Create the XDM 0.9 based speech decoder */
    hSd = Sdec_create(hEngine, args->codecName, &params, &dynParams);

    if (hSd == NULL) {
        printf("Failed to create speech decoder\n");
        goto cleanup;
    }

    lAttrs.readSize = BUFSIZE;
    lAttrs.readBufSize = 1 * 1024 * 1024;

    /* Create the file loader */
    hLoader = Loader_create(args->inFile, &lAttrs);

    if (hLoader == NULL) {
        printf("Failed to create loader\n");
        goto cleanup;
    }

    hOutBuf = Buffer_create(BUFSIZE * 8, &bAttrs); // TODO ask codec

    /* Prime the file loader */
    Loader_prime(hLoader, &hInBuf);

    while (1) {
        /* Decode the video buffer */
        if (Sdec_process(hSd, hInBuf, hOutBuf) < 0) {
            printf("Failed to decode video buffer\n");
            goto cleanup;
        }

        if (Sound_write(hSound, hOutBuf) < 0) {
            printf("Failed to write audio buffer\n");
            goto cleanup;
        }

        /* Load a new frame from the file system */
        Loader_getFrame(hLoader, hInBuf);

        if (Buffer_getUserPtr(hInBuf) == NULL) {
            printf("No more data available.\n");
            break;
        }
    }

cleanup:
    /* Clean up the application */
    if (hLoader) {
        Loader_delete(hLoader);
    }

    if (hSd) {
        Sdec_delete(hSd);
    }

    if (hOutBuf) {
        Buffer_delete(hOutBuf);
    }

    if (hSound) {
        Sound_delete(hSound);
    }

    if (hEngine) {
        Engine_close(hEngine);
    }

    return;
}
