/* --COPYRIGHT--,BSD
 * Copyright (c) $(CPYYEAR), Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

#include <stdio.h>

#include <unistd.h>
#include <pthread.h>
#include <sys/resource.h>

#include <xdc/std.h>

#include "../appMain.h"

void *thr2Fxn(void *arg);

/******************************************************************************
 * main
 ******************************************************************************/
Int main(Int argc, Char *argv[])
{   
    /* Set the priority of this whole process to max (requires root) */
    setpriority(PRIO_PROCESS, 0, -20);
    
    appMain();

    return 0;
}

/******************************************************************************
 * thr2Fxn
 ******************************************************************************/
void *thr2Fxn(void *arg)
{
    thr2Main();
    
    return 0;
}

/******************************************************************************
 * thr3Fxn
 ******************************************************************************/
void *thr3Fxn(void *arg)
{
    thr3Main();
    
    return 0;
}

/******************************************************************************
 * createThread2
 ******************************************************************************/
Bool createThread2()
{
    struct sched_param schedParam;
    pthread_t          thread2;
    pthread_attr_t     attr;
    
    /* Initialize the thread attributes */
    if (pthread_attr_init(&attr)) {
        printf("Failed to initialize thread attrs\n");
        return FALSE;
    }

    /* Force the thread to use custom scheduling attributes */
    if (pthread_attr_setinheritsched(&attr, PTHREAD_EXPLICIT_SCHED)) {
        printf("Failed to set schedule inheritance attribute\n");
        return FALSE;
    }

    /* Set the thread to be fifo real time scheduled */
    if (pthread_attr_setschedpolicy(&attr, SCHED_FIFO)) {
        printf("Failed to set FIFO scheduling policy\n");
        return FALSE;
    }
    
    /* Set thread 2 priority */
    schedParam.sched_priority = sched_get_priority_max(SCHED_FIFO) - 1;
    if (pthread_attr_setschedparam(&attr, &schedParam)) {
        printf("Failed to set scheduler parameters\n");
        return FALSE;
    }

    /* Create thread 2 */
    if (pthread_create(&thread2, &attr, thr2Fxn, NULL)) {
        printf("Failed to create display thread\n");
        return FALSE;
    }
    
    return TRUE;
}

/******************************************************************************
 * createThread3
 ******************************************************************************/
Bool createThread3()
{
    struct sched_param schedParam;
    pthread_t          thread3;
    pthread_attr_t     attr;
    
    /* Initialize the thread attributes */
    if (pthread_attr_init(&attr)) {
        printf("Failed to initialize thread attrs\n");
        return FALSE;
    }

    /* Force the thread to use custom scheduling attributes */
    if (pthread_attr_setinheritsched(&attr, PTHREAD_EXPLICIT_SCHED)) {
        printf("Failed to set schedule inheritance attribute\n");
        return FALSE;
    }

    /* Set the thread to be fifo real time scheduled */
    if (pthread_attr_setschedpolicy(&attr, SCHED_FIFO)) {
        printf("Failed to set FIFO scheduling policy\n");
        return FALSE;
    }
    
    /* Set thread 3 priority */
    schedParam.sched_priority = sched_get_priority_max(SCHED_FIFO);
    if (pthread_attr_setschedparam(&attr, &schedParam)) {
        printf("Failed to set scheduler parameters\n");
        return FALSE;
    }

    /* Create thread 3 */
    if (pthread_create(&thread3, &attr, thr3Fxn, NULL)) {
        printf("Failed to create display thread\n");
        return FALSE;
    }
    
    return TRUE;
}

/******************************************************************************
 * sleepForOneSecond
 ******************************************************************************/
Void sleepForOneSecond()
{
    usleep(1000000);
}
