/* --COPYRIGHT--,BSD
 * Copyright (c) $(CPYYEAR), Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <unistd.h>
#include <string.h>

#include <xdc/std.h>

#include <ti/sdo/ce/Engine.h>
#include <ti/sdo/ce/CERuntime.h>

#include <ti/sdo/dmai/Dmai.h>
#include <ti/sdo/dmai/Ccv.h>
#include <ti/sdo/dmai/Cpu.h>
#include <ti/sdo/dmai/Time.h>
#include <ti/sdo/dmai/BufTab.h>
#include <ti/sdo/dmai/Capture.h>
#include <ti/sdo/dmai/Framecopy.h>
#include <ti/sdo/dmai/BufferGfx.h>
#include <ti/sdo/dmai/ce/Venc.h>
#include <ti/sdo/dmai/ce/Venc1.h>

#include "appMain.h"

#define NUM_BUFS                2
#define NUM_CAPTURE_BUFS        3

/******************************************************************************
 * appMain
 ******************************************************************************/
Void appMain(Args * args)
{
    VIDENC_Params           params      = Venc_Params_DEFAULT;
    VIDENC_DynamicParams    dynParams   = Venc_DynamicParams_DEFAULT;
    VIDENC1_Params          params1     = Venc1_Params_DEFAULT;
    VIDENC1_DynamicParams   dynParams1  = Venc1_DynamicParams_DEFAULT;
    BufferGfx_Attrs         gfxAttrs    = BufferGfx_Attrs_DEFAULT;
    Ccv_Attrs               ccvAttrs    = Ccv_Attrs_DEFAULT;
    Framecopy_Attrs         fcAttrs     = Framecopy_Attrs_DEFAULT;
    Time_Attrs              tAttrs      = Time_Attrs_DEFAULT;
    Ccv_Handle              hCcv        = NULL;
    Framecopy_Handle        hFc         = NULL;
    Venc_Handle             hVe         = NULL;
    Venc1_Handle            hVe1        = NULL;
    Capture_Handle          hCapture    = NULL;
    FILE                   *outFile     = NULL;
    Engine_Handle           hEngine     = NULL;
    BufTab_Handle           hCapBufTab  = NULL;
    Time_Handle             hTime       = NULL;
    Buffer_Handle           hOutBuf     = NULL;
    Buffer_Handle           hInBuf      = NULL;
    Int                     numFrame    = 0;
    Capture_Attrs           cAttrs;
    Buffer_Handle           hCapBuf;
    Int                     bufSize, inBufSize, outBufSize;
    Cpu_Device              device;
    VideoStd_Type           videoStd;
    UInt32                  time;

    /* Initialize the codec engine run time */
    CERuntime_init();
    
    /* Initialize DMAI */
    Dmai_init();
    
    if (Cpu_getDevice(NULL, &device) < 0) {
        printf("Failed to determine target board\n");
        goto cleanup;
    }

    if (device == Cpu_Device_DM6467) {
        cAttrs = Capture_Attrs_DM6467_DEFAULT;
    }
    else {
        cAttrs = Capture_Attrs_DM6446_DM355_DEFAULT;
    }

    if (args->benchmark) {
        hTime = Time_create(&tAttrs);

        if (hTime == NULL) {
            printf("Failed to create Time object\n");
            goto cleanup;
        }
    }

    outFile = fopen(args->outFile, "wb");

    if (outFile == NULL) {
        printf("Failed to open output file %s\n", args->outFile);
        goto cleanup;
    }

    cAttrs.videoInput = args->videoInput;
    if (args->captureAlloc) {
        hCapture = Capture_create(NULL, &cAttrs);

        if (hCapture == NULL) {
            printf("Failed to create capture device\n");
            goto cleanup;
        }

        videoStd = Capture_getVideoStd(hCapture);
    }
    else {
        if (Capture_detectVideoStd(NULL, &videoStd, &cAttrs) < 0) {
            printf("Failed to detect capture video standard\n");
            goto cleanup;
        }

        gfxAttrs.colorSpace = device == Cpu_Device_DM6467 ?
                    ColorSpace_YUV422PSEMI : ColorSpace_UYVY;

        if (BufferGfx_calcDimensions(videoStd, gfxAttrs.colorSpace,
                                     &gfxAttrs.dim) < 0) {
            printf("Failed to calculate dimensions for display buffers\n");
            goto cleanup;
        }

        bufSize = BufferGfx_calcSize(videoStd, gfxAttrs.colorSpace);

        if (bufSize < 0) {
            printf("Failed to calculated size for display buffers\n");
            goto cleanup;
        }

        hCapBufTab = BufTab_create(NUM_CAPTURE_BUFS, bufSize,
                                   BufferGfx_getBufferAttrs(&gfxAttrs));

        if (hCapBufTab == NULL) {
            printf("Failed to allocate contiguous buffers\n");
            goto cleanup;
        }

        hCapture = Capture_create(hCapBufTab, &cAttrs);

        if (hCapture == NULL) {
            printf("Failed to create capture device\n");
            goto cleanup;
        }
    }

    gfxAttrs.colorSpace = device == Cpu_Device_DM6467 ? ColorSpace_YUV420PSEMI :
                                                        ColorSpace_UYVY;

    if (BufferGfx_calcDimensions(videoStd, gfxAttrs.colorSpace,
                                 &gfxAttrs.dim) < 0) {
        printf("Failed to calculate dimensions for display buffers\n");
        goto cleanup;
    }

    /* Open the codec engine */
    hEngine = Engine_open(args->engineName, NULL, NULL);

    if (hEngine == NULL) {
        printf("Failed to open codec engine: %s\n", args->engineName);
        goto cleanup;
    }

    if (args->vXdm == Xdm0) {
        if (VideoStd_getResolution(Capture_getVideoStd(hCapture),
                                   &params1.maxWidth,
                                   &params1.maxHeight) < 0) {
            printf("Invalid video standard (%d)\n",
                   Capture_getVideoStd(hCapture));
            goto cleanup;
        }

        params.rateControlPreset =
            args->bitRate < 0 ? IVIDEO_NONE : IVIDEO_LOW_DELAY;

        params.maxBitRate =
            args->bitRate < 0 ? 0 : args->bitRate;

        if (device != Cpu_Device_DM6467) {
           params.inputChromaFormat = XDM_YUV_422ILE;
        }

        dynParams.targetBitRate =
            args->bitRate < 0 ? 0 : args->bitRate;

        dynParams.inputWidth = params.maxWidth;
        dynParams.inputHeight = params.maxHeight;
        /* Create the video encoder */
        hVe = Venc_create(hEngine, args->codecName, &params, &dynParams);

        if (hVe == NULL) {
            printf("Failed to create video encoder: %s\n", args->codecName);
            goto cleanup;
        }

        inBufSize = Venc_getInBufSize(hVe);
        outBufSize = Venc_getOutBufSize(hVe);
    }
    else {
        if (VideoStd_getResolution(Capture_getVideoStd(hCapture),
                                   &params1.maxWidth,
                                   &params1.maxHeight) < 0) {
            printf("Invalid video standard (%d)\n",
                   Capture_getVideoStd(hCapture));
            goto cleanup;
        }

        params1.rateControlPreset =
            args->bitRate < 0 ? IVIDEO_NONE : IVIDEO_LOW_DELAY;

        params1.maxBitRate =
            args->bitRate < 0 ? 0 : args->bitRate;

        if (device != Cpu_Device_DM6467) {
           params1.inputChromaFormat = XDM_YUV_422ILE;
           params1.reconChromaFormat = XDM_YUV_420P;
        }

        dynParams1.targetBitRate =
            args->bitRate < 0 ? 0 : args->bitRate;

        dynParams1.inputWidth = params1.maxWidth;
        dynParams1.inputHeight = params1.maxHeight;

        /* Create the video encoder */
        hVe1 = Venc1_create(hEngine, args->codecName, &params1, &dynParams1);

        if (hVe1 == NULL) {
            printf("Failed to create video encoder: %s\n", args->codecName);
            goto cleanup;
        }

        outBufSize = Venc1_getOutBufSize(hVe1);
        inBufSize = Venc1_getInBufSize(hVe1);
    }

    hInBuf = Buffer_create(inBufSize, BufferGfx_getBufferAttrs(&gfxAttrs));

    if (hInBuf == NULL) {
        printf("Failed to allocate contiguous buffer\n");
        goto cleanup;
    }

    hOutBuf = Buffer_create(outBufSize, BufferGfx_getBufferAttrs(&gfxAttrs));

    if (hOutBuf == NULL) {
        printf("Failed to create contiguous buffer\n");
        goto cleanup;
    }

    if (device == Cpu_Device_DM6467) {
        ccvAttrs.accel = args->accel;

        hCcv = Ccv_create(&ccvAttrs);

        if (hCcv == NULL) {
            printf("Failed to create color conversion job\n");
            goto cleanup;
        }

        if (Ccv_config(hCcv, BufTab_getBuf(Capture_getBufTab(hCapture), 0),
                       hInBuf) < 0) {
            printf("Failed to configure color conversion job\n");
            goto cleanup;
        }
    }
    else {
        fcAttrs.accel = args->accel;

        hFc = Framecopy_create(&fcAttrs);

        if (hFc == NULL) {
            printf("Failed to configure frame copy job\n");
            goto cleanup;
        }

        if (Framecopy_config(hFc,
                             BufTab_getBuf(Capture_getBufTab(hCapture), 0),
                             hInBuf) < 0) {
            printf("Failed to configure frame copy job\n");
            goto cleanup;
        }
    }

    while (numFrame++ < args->numFrames) {
        /* Get a buffer from the display driver */
        if (Capture_get(hCapture, &hCapBuf) < 0) {
            printf("Failed to get capture buffer\n");
            goto cleanup;
        }

        if (args->benchmark) {
            if (Time_reset(hTime) < 0) {
                printf("Failed to reset timer\n");
                goto cleanup;
            }
        }

        if (device == Cpu_Device_DM6467) {
            if (Ccv_execute(hCcv, hCapBuf, hInBuf) < 0) {
                printf("Failed to execute color conversion job\n");
                goto cleanup;
            }
        }
        else {
            if (Framecopy_execute(hFc, hCapBuf, hInBuf) < 0) {
                printf("Failed to execute frame copy job\n");
                goto cleanup;
            }
        }

        if (args->benchmark) {
            if (Time_delta(hTime, &time) < 0) {
                printf("Failed to get timer delta\n");
                goto cleanup;
            }

            printf("%d: CCV / Framecopy: %uus ", numFrame, (Uns) time);
        }

        if (args->vXdm == Xdm0) {
            /* Decode the video buffer */
            if (Venc_process(hVe, hInBuf, hOutBuf) < 0) {
                printf("Failed to encode video buffer\n");
                goto cleanup;
            }
        }
        else {
            /* Decode the video buffer */
            if (Venc1_process(hVe1, hInBuf, hOutBuf) < 0) {
                printf("Failed to encode video buffer\n");
                goto cleanup;
            }
        }

        if (args->benchmark) {
            if (Time_delta(hTime, &time) < 0) {
                printf("Failed to get timer delta\n");
                goto cleanup;
            }

            printf("Encode: %uus ", (Uns) time);
        }

        /* Give capture buffer back to capture device driver */
        if (Capture_put(hCapture, hCapBuf) < 0) {
            printf("Failed to put capture buffer\n");
            goto cleanup;
        }

        if (args->benchmark) {
            if (Time_total(hTime, &time) < 0) {
                printf("Failed to get timer total\n");
                goto cleanup;
            }

            printf("Total: %uus\n", (Uns) time);
        }

        /* Write the encoded frame to the file system */
        if ((Buffer_getNumBytesUsed(hOutBuf) != 0) && 
            (fwrite(Buffer_getUserPtr(hOutBuf), Buffer_getNumBytesUsed(hOutBuf),
             1, outFile) != 1)) {
            printf("Failed to write encoded video data to disc\n");
            goto cleanup;
        }
    }

cleanup:
    /* Clean up the application */
    if (hCapBufTab) {
        BufTab_delete(hCapBufTab);
    }

    if (hOutBuf) {
        Buffer_delete(hOutBuf);
    }

    if (hInBuf) {
        Buffer_delete(hInBuf);
    }

    if (hVe) {
        Venc_delete(hVe);
    }

    if (hVe1) {
        Venc1_delete(hVe1);
    }

    if (hFc) {
        Framecopy_delete(hFc);
    }

    if (hCcv) {
        Ccv_delete(hCcv);
    }

    if (hCapture) {
        Capture_delete(hCapture);
    }

    if (hEngine) {
        Engine_close(hEngine);
    }

    if (outFile) {
        fclose(outFile);
    }

    if (hTime) {
        Time_delete(hTime);
    }

    return;
}
