/* --COPYRIGHT--,BSD
 * Copyright (c) $(CPYYEAR), Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

#include <stdio.h>
#include <string.h>
#include <unistd.h>

#include <xdc/std.h>

#include <ti/sdo/dmai/Cpu.h>
#include <ti/sdo/dmai/Dmai.h>
#include <ti/sdo/dmai/Time.h>
#include <ti/sdo/dmai/Display.h>
#include <ti/sdo/dmai/BufferGfx.h>

#include <ti/sdo/dmai/Resize.h>

#include "appMain.h"

static unsigned int ycbcr [] = {
    0x00000000,
    0xBF80BF80,
    0x10801080,
    0x00000000,
    0xBF80BF80,
    0x00000000,
    0x68C868DB,
    0xBF80BF80,
};

Void fillColorBar (Buffer_Handle hBuf)
{
    Int i, pos, j;
    Int32 * bufPtr = (Int32*) Buffer_getUserPtr (hBuf);
    Int32 bufSize = Buffer_getSize (hBuf) / sizeof(Int32);

    for (i=0; i < 8; i++) {
        pos = i * (bufSize / 8);
        for (j=0; j < bufSize / 8; j++) 
            bufPtr[pos+j] = ycbcr[i];
    }
    
}
/******************************************************************************
 * appMain
 *****************************************************************************/
Int appMain (Args *argsp)
{
    BufferGfx_Attrs         gfxAttrs = BufferGfx_Attrs_DEFAULT;
    Resize_Attrs            rszAttrs = Resize_Attrs_DEFAULT;
    Buffer_Handle           hInBuf = NULL, hOutBuf = NULL;
    Resize_Handle           hResize = NULL;
    Cpu_Attrs               cpuAttrs = Cpu_Attrs_DEFAULT;
    Cpu_Handle              hCpu = NULL;
    Time_Attrs              timeAttrs = Time_Attrs_DEFAULT;
    Time_Handle             hTime = NULL;
    Int32                   hInBufSize,hOutBufSize;
    Int                     inLoop, outLoop;
    FILE                    *inFile = NULL, *outFile = NULL;
    Int                     ret = Dmai_EOK;
    Uint32                  totalTime;
    Int                     cpuLoad;
 
    /* Initialize DMAI */
    Dmai_init();

    if (argsp->benchmark) {
        hCpu = Cpu_create (&cpuAttrs);
    
        if (hCpu == NULL) {
        fprintf (stderr, "Failed to create cpu object\n");
            ret = Dmai_EFAIL;
            goto cleanup;
        }
            
        hTime = Time_create (&timeAttrs);

        if (hTime == NULL) {
            fprintf (stderr, "Failed to create time object\n");
            ret = Dmai_EFAIL;
            goto cleanup;
        }
    }

    if (hTime)
        Time_reset (hTime);
    
    /* Open resizer module */
    hResize = Resize_create(&rszAttrs);
    
    if (hResize == NULL) {
        fprintf (stderr, "Failed to open resizer module.\n");
        ret = Dmai_EFAIL;
        goto cleanup;
    }

    outLoop = 0;

    do {

        inLoop = 1;
        inFile = outFile  = NULL;

        if (argsp->inFile != NULL) {
            inFile = fopen (argsp->inFile, "rb");

            if (inFile == NULL) {
                fprintf (stderr, "Failed to open %s\n", argsp->inFile);
                ret = Dmai_EFAIL;
                goto cleanup;
            }
        }

        if (argsp->outFile != NULL) {
            outFile = fopen (argsp->outFile, "wb");

            if (outFile == NULL) {
                fprintf (stderr, "Failed to open %s\n", argsp->outFile);
                ret = Dmai_EFAIL;
                goto cleanup;
            }
        }

        /* Input buffer */
        gfxAttrs.dim.width      = argsp->srcWidth;
        gfxAttrs.dim.height     = argsp->srcHeight;
        gfxAttrs.dim.lineLength = argsp->srcWidth * 2;
        hInBufSize = gfxAttrs.dim.lineLength * gfxAttrs.dim.height;

        hInBuf =Buffer_create(hInBufSize, BufferGfx_getBufferAttrs(&gfxAttrs));

        if (hInBuf == NULL) {
            printf ("Failed to allocate memory for input buffer.\n");
            ret = Dmai_EFAIL;
            goto cleanup;
        }
    
        /* Output buffer */
        gfxAttrs.dim.width      = argsp->dstWidth;
        gfxAttrs.dim.height     = argsp->dstHeight;
        gfxAttrs.dim.lineLength = argsp->dstWidth * 2;
        hOutBufSize  = gfxAttrs.dim.lineLength * gfxAttrs.dim.height;

        hOutBuf=Buffer_create(hOutBufSize,BufferGfx_getBufferAttrs(&gfxAttrs));

        if (hOutBuf == NULL) {
            printf ("Failed to allocate memory for output buffer.\n");
            ret = Dmai_EFAIL;
            goto cleanup;
        }

        printf ("Resize %dx%d - %dx%d (%.2fx%.2f)\n", 
                  argsp->srcWidth, argsp->srcHeight, argsp->dstWidth, 
                  argsp->dstHeight, (Float) argsp->dstWidth/argsp->srcWidth,
                  (Float) argsp->dstHeight/argsp->srcHeight);
 
        /* Configure resizer */
        if (Resize_config(hResize, hInBuf, hOutBuf) < 0) {
            printf ("Failed to configure resizer.\n");
            ret = Dmai_EFAIL;
            goto cleanup;
        }

        if (inFile == NULL)  
            fillColorBar (hInBuf);

        do {

            printf ("Processing frame %d(%d)    \r", inLoop, argsp->numFrames);
            fflush(stdout);

            if (inFile != NULL)  
                fread (Buffer_getUserPtr(hInBuf), hInBufSize, 1, inFile);
        
            if (Resize_execute(hResize, hInBuf, hOutBuf) < 0) {
                printf ("Failed to execute resizer.\n");
                ret = Dmai_EFAIL;
                goto cleanup;
            }

            if(outFile != NULL) 
                fwrite (Buffer_getUserPtr(hOutBuf), hOutBufSize, 1, outFile);

            if (inFile != NULL)
                if (feof(inFile))
                    break;

        }while(inLoop++ < argsp->numFrames);

        printf ("\n");

        if (outFile != NULL) {
            fclose (outFile);
        }

        if (inFile != NULL) 
            fclose (inFile);

        if (!(inFile || outFile)) {
            argsp->srcWidth = argsp->srcWidth + 32;
            argsp->srcHeight = argsp->srcHeight + 32 ;
            argsp->dstWidth = argsp->dstWidth - 32;
            argsp->dstHeight = argsp->dstHeight - 32;
        }

        Buffer_delete(hInBuf);
        Buffer_delete(hOutBuf);
        hInBuf = hOutBuf = NULL;

    }while(outLoop++ < 10 && argsp->loop);

    if (hCpu)
        Cpu_getLoad (hCpu, &cpuLoad);

    if (hTime)
        Time_total (hTime, &totalTime);
 
    if (argsp->benchmark) { 
        printf ("cpu load: %d\n", cpuLoad);
        printf ("time : %lu us , %.2f s\n", totalTime, 
                    (Float) totalTime/1000000);
    }

cleanup:

    if (hResize)    
        Resize_delete (hResize);

    if (hInBuf)
        Buffer_delete (hInBuf);
    
    if (hOutBuf)
        Buffer_delete (hOutBuf);
    
    if (inFile)
        fclose (inFile);

    if (outFile)
        fclose (outFile);

    if (hTime)
        Time_delete (hTime);

    if (hCpu)
        Cpu_delete (hCpu);

    return ret;
}
 
