/* --COPYRIGHT--,BSD
 * Copyright (c) $(CPYYEAR), Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * --/COPYRIGHT--*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <getopt.h>

#include <xdc/std.h>

#include "../appMain.h"

/* Default arguments for app */
#define DEFAULT_ARGS { 320,240,640,480, 1000, FALSE, NULL, NULL, FALSE}

/*
 * Argument IDs for long options. They must not conflict with ASCII values,
 * so start them at 256.
 */
typedef enum
{
   ArgID_INPUT_RES = 256,
   ArgID_NUMFRAMES,
   ArgID_OUTPUT_RES,
   ArgID_INFILE,
   ArgID_OUTFILE,
   ArgID_BENCHMARK,
   ArgID_HELP,
   ArgID_LOOP,
   ArgID_NUMARGS
} ArgID;

/******************************************************************************
 * usage
 ******************************************************************************/
static Void usage(void)
{
    fprintf (stderr, "Usage video_resize_<platform> [options]\n\n"
     "Options:\n"
     "      --infile         Input file name\n"
     "      --outfile        Output file name\n"
     "      --loop           Execute the test loop\n"
     "      --benchmark      Print benchmarking information\n"
     "-i  | --input          Input video resolution [Default:320x240]\n"
     "-o  | --output         Output video resolution [Default:640x480]\n"
     "-n  | --numframes      Number of frames [Default:60]\n");
}

/******************************************************************************
 * parseArgs
 *****************************************************************************/
static Void parseArgs(Int argc, Char *argv[], Args *argsp)
{
    const char shortOptions[] = "i:o:n:h";

    const struct option longOptions[] = {
        {"input",   required_argument,  NULL, ArgID_INPUT_RES},
        {"output",   required_argument,  NULL, ArgID_OUTPUT_RES},
        {"infile",   required_argument,  NULL, ArgID_INFILE},
        {"outfile",   required_argument,  NULL, ArgID_OUTFILE},
        {"numframes",   required_argument,  NULL, ArgID_NUMFRAMES},
        {"loop",   no_argument,  NULL, ArgID_LOOP},
        {"benchmark", no_argument,  NULL, ArgID_BENCHMARK},
        {"help" ,   no_argument, NULL, ArgID_HELP},
        {0,0,0,0}
    };

    Int index;
    Int argID;

    for (;;) {
        argID = getopt_long(argc, argv, shortOptions, longOptions, &index);

        if (argID == -1) {
            break;
        }

        switch (argID) {

            case ArgID_INPUT_RES:
            case 'i':
                    if (sscanf(optarg,"%dx%d",&argsp->srcWidth,
                            &argsp->srcHeight) != 2) {
                        fprintf (stderr, "Invalid input resolution\n");
                        usage ();
                        exit (EXIT_FAILURE);
                    }
                    break;

            case ArgID_OUTPUT_RES:
            case 'o':
                    if (sscanf(optarg,"%dx%d",&argsp->dstWidth,
                            &argsp->dstHeight) != 2) {
                        fprintf (stderr, "Invalid output resolution\n");
                        usage ();
                        exit (EXIT_FAILURE);
                    }
                    break;

             case ArgID_INFILE:
                    argsp->inFile = optarg;
                    break;
         
             case ArgID_OUTFILE:
                    argsp->outFile = optarg;
                    break;

             case ArgID_NUMFRAMES:
             case 'n':
                    argsp->numFrames = atoi (optarg);
                    break;

             case ArgID_HELP:
             case 'h':
                    usage ();
                    exit (EXIT_SUCCESS);
                    break;

             case ArgID_BENCHMARK:
                    argsp->benchmark = TRUE;
                    break;

             case ArgID_LOOP:
                    argsp->loop = TRUE;
                    break;

             default:
                    usage ();
                    exit (EXIT_FAILURE);
        }
    } 

    if (optind < argc) {
         usage ();
         exit (EXIT_FAILURE);
    }
}


/******************************************************************************
 * main
 *****************************************************************************/
Int main (Int argc, Char *argv[])
{
    Args args = DEFAULT_ARGS;

    /* Parse the argument given to the app */
    parseArgs(argc, argv, &args);

    return appMain(&args);
}

